@@ -1,31 +1,42 @@
 # -*- coding: utf-8 -*-
-# :Progetto:  SoL -- Business objects
-# :Creato:    sab 27 set 2008 14:00:47 CEST
-# :Autore:    Lele Gaifax <lele@metapensiero.it>
-# :Licenza:   GNU General Public License version 3 or later
+# :Project:   SoL -- Business objects
+# :Created:   sab 27 set 2008 14:00:47 CEST
+# :Author:    Lele Gaifax <lele@metapensiero.it>
+# :License:   GNU General Public License version 3 or later
+# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2018, 2020 Lele Gaifax
 #
 
 """The application's model objects"""
 
+from datetime import datetime
+import logging
 from uuid import uuid1
 
-from sqlalchemy import Column, Index, func
+from sqlalchemy import Column, Index, event, func
+from sqlalchemy.engine import Engine
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.ext.hybrid import hybrid_property
-from sqlalchemy.orm import object_session, scoped_session, sessionmaker
+from sqlalchemy.orm import object_session
 
-import zope.sqlalchemy
+from ..i18n import translatable_string as _
+from .domains import date_t, guid_t, timestamp_t
 
-from ..i18n import translatable_string as N_
-from .domains import guid_t, timestamp_t
+
+logger = logging.getLogger(__name__)
 
 
 class AbstractBase(object):
     "Abstract base entity class."
 
     @classmethod
-    def check_insert(klass, session, fields):
-        "Perform any check before an new instance is inserted"
+    def check_insert(klass, session, fields, user_id):
+        """Perform any check before an new instance is inserted.
+
+        :param session: the SA session
+        :param fields: a dictionary with the fields values
+        :param user_id: either the string "admin" or the ID of the user performing the
+                        operation
+        """
 
     def check_update(self, fields):
         """Perform any check before updating the instance.
@@ -51,7 +62,7 @@ class AbstractBase(object):
         :returns: a mapping between field name and a tuple ``(oldvalue, newvalue)``,
                   for each modified field
 
-        First call :py:meth:`.check_update` to assert the validity of incoming data,
+        First call :py:meth:`check_update` to assert the validity of incoming data,
         then update the instance fields.
 
         If `missing_only` is ``True`` then only the fields that are currently `empty`
@@ -61,7 +72,11 @@ class AbstractBase(object):
         updated.
         """
 
-        self.check_update(data)
+        try:
+            self.check_update(data)
+        except Exception as e:
+            logger.error('Error trying to update %r with %r: %s', self, data, e)
+            raise
 
         changes = {}
 
@@ -73,6 +88,9 @@ class AbstractBase(object):
                             cvalue is True or cvalue is False):
                         continue
                 nvalue = data[attr]
+                if ((attr in self.__table__.c and self.__table__.c[attr].type is date_t
+                     and isinstance(nvalue, datetime))):
+                    nvalue = nvalue.date()
                 if cvalue != nvalue:
                     setattr(self, attr, nvalue)
 
@@ -116,8 +134,8 @@ class GloballyUnique(object):
         guid_t, name="guid",
         nullable=False,
         default=lambda: uuid1().hex,
-        info=dict(label=N_('GUID'),
-                  hint=N_('The globally unique identifier of the record.'),
+        info=dict(label=_('GUID'),
+                  hint=_('The globally unique identifier of the record.'),
                   hidden=True,
                   sortable=False))
     """An UUID key."""
@@ -126,8 +144,8 @@ class GloballyUnique(object):
         timestamp_t,
         nullable=False,
         server_default=func.now(),
-        info=dict(label=N_('Modified'),
-                  hint=N_('Timestamp of the last change to the record.'),
+        info=dict(label=_('Modified'),
+                  hint=_('Timestamp of the last change to the record.'),
                   hidden=True))
     """Last update timestamp."""
 
@@ -156,52 +174,46 @@ class GloballyUnique(object):
                       unique=True),)
 
 
-DBSession = scoped_session(sessionmaker())
-"The global SA session maker"
-
-zope.sqlalchemy.register(DBSession)
-
 Base = declarative_base(cls=AbstractBase)
 "The common parent class for all declarative mapped classed."
 
 
-from .championship import Championship
-from .club import Club
-from .competitor import Competitor
-from .match import Match
-from .mergedplayer import MergedPlayer
-from .player import Player
-from .rate import Rate
-from .rating import Rating
-from .tourney import Tourney
+from .board import Board               # noqa
+from .championship import Championship # noqa
+from .club import Club                 # noqa
+from .competitor import Competitor     # noqa
+from .match import Match               # noqa
+from .mergedplayer import MergedPlayer # noqa
+from .player import Player             # noqa
+from .rate import Rate                 # noqa
+from .rating import Rating             # noqa
+from .tourney import Tourney           # noqa
+from .user import User                 # noqa
 
 
-def wipe_database():
+def wipe_database(session):
     """Delete all records in all tables."""
 
     import transaction
     from zope.sqlalchemy import mark_changed
 
-    s = DBSession()
-
     with transaction.manager:
-        s.execute(MergedPlayer.__table__.delete())
-        s.execute(Match.__table__.delete())
-        s.execute(Competitor.__table__.delete())
-        s.execute(Rate.__table__.delete())
-        s.execute(Player.__table__.delete())
-        s.execute(Tourney.__table__.delete())
-        s.execute(Rating.__table__.delete())
-        s.execute(Championship.__table__.delete())
-        s.execute(Club.__table__.delete())
-        mark_changed(s)
+        session.execute(MergedPlayer.__table__.delete())
+        session.execute(Board.__table__.delete())
+        session.execute(Match.__table__.delete())
+        session.execute(Competitor.__table__.delete())
+        session.execute(Rate.__table__.delete())
+        session.execute(Player.__table__.delete())
+        session.execute(Tourney.__table__.delete())
+        session.execute(Championship.__table__.delete())
+        session.execute(Club.__table__.delete())
+        session.execute(Rating.__table__.delete())
+        session.execute(User.__table__.delete())
+        mark_changed(session)
 
 
 # Activate FOREIGN KEYS
 
-from sqlalchemy.engine import Engine
-from sqlalchemy import event
-
 @event.listens_for(Engine, "connect")
 def set_sqlite_pragma(dbapi_connection, connection_record):
     cursor = dbapi_connection.cursor()
