@@ -1,17 +1,18 @@
 # -*- coding: utf-8 -*-
-# :Progetto:  SoL -- The Championship entity
-# :Creato:    gio 27 nov 2008 13:53:28 CET
-# :Autore:    Lele Gaifax <lele@metapensiero.it>
-# :Licenza:   GNU General Public License version 3 or later
+# :Project:   SoL -- The Championship entity
+# :Created:   gio 27 nov 2008 13:53:28 CET
+# :Author:    Lele Gaifax <lele@metapensiero.it>
+# :License:   GNU General Public License version 3 or later
+# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020 Lele Gaifax
 #
 
 import logging
 
-from sqlalchemy import Column, ForeignKey, Index, Sequence
+from sqlalchemy import Column, ForeignKey, Index, Sequence, exists, or_, select
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import backref, relationship
 
-from ..i18n import translatable_string as N_, gettext as _
+from ..i18n import gettext, translatable_string as _
 from . import Base, GloballyUnique
 from .domains import (
     boolean_t,
@@ -20,6 +21,8 @@ from .domains import (
     intid_t,
     smallint_t,
     )
+from .errors import OperationAborted
+from .utils import normalize
 
 
 logger = logging.getLogger(__name__)
@@ -44,47 +47,58 @@ class Championship(GloballyUnique, Base):
         intid_t, Sequence('gen_idchampionship', optional=True),
         primary_key=True,
         nullable=False,
-        info=dict(label=N_('Championship ID'),
-                  hint=N_('Unique ID of the championship.')))
+        info=dict(label=_('Championship ID'),
+                  hint=_('Unique ID of the championship.')))
     """Primary key."""
 
     idprevious = Column(
-        intid_t, ForeignKey('championships.idchampionship'),
-        info=dict(label=N_('Previous championship ID'),
-                  hint=N_('ID of the previous championship.')))
+        intid_t, ForeignKey('championships.idchampionship', name='fk_championship_previous'),
+        info=dict(label=_('Previous championship ID'),
+                  hint=_('ID of the previous championship.')))
     """Previous championship's ID."""
 
     idclub = Column(
-        intid_t, ForeignKey('clubs.idclub'), nullable=False,
-        info=dict(label=N_('Club ID'),
-                  hint=N_('ID of the club the championship is organized by.')))
+        intid_t, ForeignKey('clubs.idclub', name='fk_championship_club'),
+        nullable=False,
+        info=dict(label=_('Club ID'),
+                  hint=_('ID of the club the championship is organized by.')))
     """Organizer :py:class:`club <.Club>`'s ID."""
 
+    idrating = Column(
+        intid_t, ForeignKey('ratings.idrating', name='fk_championship_rating',
+                            ondelete="SET NULL"),
+        nullable=True,
+        info=dict(label=_('Rating ID'),
+                  hint=_('ID of the default rating used by tourneys in this championship.')))
+    """Possible :py:class:`rating <.Rating>` ID, used as default value
+    for the corresponding field when creating a new tourney."""
+
     idowner = Column(
-        intid_t, ForeignKey('players.idplayer', ondelete="SET NULL"),
-        info=dict(label=N_('Owner ID'),
-                  hint=N_('ID of the user that is responsible for this record.')))
-    """ID of the :py:class:`user <.Player>` that is responsible for this record."""
+        intid_t, ForeignKey('users.iduser', name='fk_championship_owner',
+                            ondelete="SET NULL"),
+        info=dict(label=_('Owner ID'),
+                  hint=_('ID of the user that is responsible for this record.')))
+    """ID of the :py:class:`user <.User>` that is responsible for this record."""
 
     description = Column(
         description_t,
         nullable=False,
-        info=dict(label=N_('Championship'),
-                  hint=N_('Description of the championship.')))
+        info=dict(label=_('Description'),
+                  hint=_('Description of the championship.')))
     """Description of the championship."""
 
     prizes = Column(
         code_t,
         nullable=False,
         default='fixed',
-        info=dict(label=N_('Prizes'),
-                  hint=N_('Method used to assign final prizes.'),
+        info=dict(label=_('Prizes'),
+                  hint=_('Method used to assign final prizes.'),
                   dictionary=dict(
-                      asis=N_('No final prizes'),
-                      fixed=N_('Fixed prizes: 18,16,14,13…'),
-                      fixed40=N_('Fixed prizes: 1000,900,800,750…'),
-                      millesimal=N_('Classic millesimal prizes'),
-                      centesimal=N_('Centesimal prizes'))))
+                      asis=_('Simple tourneys, no special prizes'),
+                      fixed=_('Fixed prizes: 18,16,14,13…'),
+                      fixed40=_('Fixed prizes: 1000,900,800,750…'),
+                      millesimal=_('Classic millesimal prizes'),
+                      centesimal=_('Centesimal prizes'))))
     """Kind of prize-giving.
 
     This is used to determine which method will be used to assign
@@ -156,9 +170,9 @@ class Championship(GloballyUnique, Base):
         smallint_t,
         nullable=False,
         default=0,
-        info=dict(label=N_('Skip'),
-                  hint=N_('Number of worst results to skip in computing'
-                          ' the final ranking of the championship.'),
+        info=dict(label=_('Skip'),
+                  hint=_('Number of worst results to skip in computing'
+                         ' the final ranking of the championship.'),
                   min=0, max=5))
     """Number of worst results to skip in computing the ranking."""
 
@@ -166,8 +180,8 @@ class Championship(GloballyUnique, Base):
         smallint_t,
         nullable=False,
         default=1,
-        info=dict(label=N_('Players'),
-                  hint=N_('Number of players in each team, 1 for singles, 2 for doubles.'),
+        info=dict(label=_('Players'),
+                  hint=_('Number of players in each team, 1 for singles, 2 for doubles.'),
                   min=1, max=4))
     """Number of players per team."""
 
@@ -175,31 +189,42 @@ class Championship(GloballyUnique, Base):
         boolean_t,
         nullable=False,
         default=False,
-        info=dict(label=N_('Closed'),
-                  hint=N_('Should be activated once the championship'
-                          ' has been completed and no other'
-                          ' tourney can be associated with it.')))
-    """Whether the championships is close, and its ranking finalized."""
+        info=dict(label=_('Closed'),
+                  hint=_('Should be activated once the championship'
+                         ' has been completed and no other'
+                         ' tourney can be associated with it.')))
+    """Whether the championships is closed, and its ranking finalized."""
 
     couplings = Column(
         code_t,
         default='serial',
-        info=dict(label=N_('Pairings'),
-                  hint=N_('Default method used to pair competitors at each round.'),
-                  dictionary=dict(serial=N_('Ranking order'),
-                                  dazed=N_('Cross ranking order'))))
+        info=dict(label=_('Pairings'),
+                  hint=_('Default method used to pair competitors at each round.'),
+                  dictionary=dict(serial=_('Ranking order'),
+                                  dazed=_('Cross ranking order'),
+                                  staggered=_('Staggered ranking order'))))
     """Kind of pairing method used to build next round, used as default value
     for the corresponding field when creating a new tourney."""
 
+    trainingboards = Column(
+        smallint_t,
+        info=dict(label=_('Training boards'),
+                  hint=_('If set, it is the number of boards to play in a “Danwei Grade”'
+                         ' competition, a.k.a. “Corona Carrom” tourneys. Leave this blank'
+                         ' for normal championships.'),
+                  min=0, max=8))
+    """When specified, this is the number of boards that players must complete in a
+    “CoronaCarrom” tourney.
+    """
+
     ## Relations
 
-    owner = relationship('Player', backref='owned_championships')
-    "The :py:class:`owner <.Player>` of this record, `admin` when ``None``."
+    owner = relationship('User', backref='owned_championships')
+    "The :py:class:`owner <.User>` of this record, `admin` when ``None``."
 
     tourneys = relationship('Tourney', backref='championship',
                             cascade="all, delete-orphan",
-                            order_by="Tourney.date",
-                            lazy=True)
+                            order_by="Tourney.date")
     """:py:class:`Tourneys <.Tourney>` in this championship."""
 
     previous = relationship('Championship', uselist=False,
@@ -207,11 +232,52 @@ class Championship(GloballyUnique, Base):
                             remote_side='Championship.idchampionship')
     """Previous championship."""
 
+    @classmethod
+    def check_insert(klass, session, fields, user_id):
+        "Check description validity"
+
+        from .club import Club, clubusers
+
+        try:
+            desc = normalize(fields['description'])
+        except KeyError:
+            raise OperationAborted(_('For a new championship the "description" field'
+                                     ' is mandatory'))
+
+        if not desc:
+            raise OperationAborted(_('For a new championship the "description" field'
+                                     ' is mandatory'))
+
+        if 'idclub' not in fields:
+            raise OperationAborted(_('For a new championship you must select an existing'
+                                     ' club'))
+
+        if user_id != 'admin':
+            clubs = Club.__table__
+            if not session.scalar(select([True])
+                                  .where(or_(
+                                      exists()
+                                      .where(clubs.c.idclub == fields['idclub'])
+                                      .where(clubs.c.idowner == user_id),
+                                      exists()
+                                      .where(clubusers.c.idclub == fields['idclub'])
+                                      .where(clubusers.c.iduser == user_id)))):
+                raise OperationAborted(_('You are not allowed to add a championship to the'
+                                         ' selected club'))
+
+    def check_update(self, fields):
+        "Check description validity"
+
+        if 'description' in fields:
+            desc = normalize(fields['description'])
+            if not desc:
+                raise OperationAborted(_('The "description" field of a championship'
+                                         ' cannot be empty'))
+
     def caption(self, html=None, localized=True):
-        return _('$championship of $club',
-                 just_subst=not localized, mapping=dict(
-                     championship=self.description,
-                     club=self.club.caption(html, localized)))
+        return gettext('$championship of $club', just_subst=not localized,
+                       mapping=dict(championship=self.description,
+                                    club=self.club.caption(html, localized)))
 
     def ranking(self, limit=None, onlywomen=False):
         """Summarize the championship, collecting final prizes of the players.
@@ -228,19 +294,19 @@ class Championship(GloballyUnique, Base):
         `limit` and `onlywomen` are used by the general rankings, to consider only last year
         tourneys and to produce women ranking respectively.
 
-        Results in a tuple of two items, the first being a list of two slots tuples,
-        respectively the date and the guid of a tourney, the second a list of tuples, sorted by
-        total prize: each tuple contains five items, a tuple of players, their total prize, a
-        list of their prizes sorted by date of event, the number of prizes and finally either
-        `None` or a list of skipped prizes.
+        Results in a tuple of two items, the first being a list of three slots tuples,
+        respectively the date, the description and the guid of a tourney, the second a list of
+        tuples, sorted by total prize: each tuple contains five items, a tuple of players,
+        their total prize, a list of their prizes sorted by date of event, the number of prizes
+        and finally either `None` or a list of skipped prizes.
         """
 
         dates = []
         allprizes = {}
         for t in self.tourneys:
-            if t.prized and (limit is None or t.date>=limit):
+            if t.prized and (limit is None or t.date >= limit):
                 prizes = t.championship.prizes
-                dates.append((t.date, t.guid))
+                dates.append((t.date, t.description, t.guid))
                 for c in t.competitors:
                     if not onlywomen or c.player1.sex == 'F':
                         players = (c.player1, c.player2, c.player3, c.player4)
@@ -253,10 +319,10 @@ class Championship(GloballyUnique, Base):
         for players in allprizes:
             prizes = allprizes[players]
             nprizes = len(prizes)
-            for d,g in dates:
-                if not d in prizes: # pragma: no cover
-                    prizes[d] = 0
-            championship.append((players, [prizes[d] for d,g in dates], nprizes))
+            for date, desc, g in dates:
+                if date not in prizes:  # pragma: no cover
+                    prizes[date] = 0
+            championship.append((players, [prizes[date] for date, desc, g in dates], nprizes))
 
         totalprizes = []
         for players, prizes, nprizes in championship:
@@ -287,15 +353,20 @@ class Championship(GloballyUnique, Base):
         simple['guid'] = self.guid
         simple['modified'] = self.modified
         simple['club'] = serializer.addClub(self.club)
+        if self.idrating:
+            simple['rating'] = serializer.addRating(self.rating)
         if self.idowner:
-            simple['owner'] = serializer.addPlayer(self.owner)
+            simple['owner'] = serializer.addUser(self.owner)
         simple['description'] = self.description
         simple['prizes'] = self.prizes
         simple['couplings'] = self.couplings
         simple['skipworstprizes'] = self.skipworstprizes
         simple['playersperteam'] = self.playersperteam
-        simple['closed'] = self.closed
+        if self.closed:
+            simple['closed'] = self.closed
         if self.previous:
             simple['previous'] = serializer.addChampionship(self.previous)
+        if self.trainingboards:
+            simple['trainingboards'] = self.trainingboards
 
         return simple
