@@ -1,21 +1,22 @@
 # -*- coding: utf-8 -*-
-# :Progetto:  SoL -- The Rating entity
-# :Creato:    gio 05 dic 2013 09:05:58 CET
-# :Autore:    Lele Gaifax <lele@metapensiero.it>
-# :Licenza:   GNU General Public License version 3 or later
+# :Project:   SoL -- The Rating entity
+# :Created:   gio 05 dic 2013 09:05:58 CET
+# :Author:    Lele Gaifax <lele@metapensiero.it>
+# :License:   GNU General Public License version 3 or later
+# :Copyright: © 2013, 2014, 2015, 2016, 2018, 2020 Lele Gaifax
 #
 
 from datetime import date
 from decimal import Decimal
 import logging
 
-from sqlalchemy import Column, ForeignKey, Index, Sequence, func, or_, select
+from sqlalchemy import Column, ForeignKey, Index, Sequence, func, or_, exists, select
 from sqlalchemy.orm import relationship
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.orm import object_session
 
-from ..i18n import translatable_string as N_
+from ..i18n import translatable_string as _
 from . import Base, GloballyUnique
 from .domains import (
     boolean_t,
@@ -27,7 +28,9 @@ from .domains import (
     smallint_t,
     volatility_t,
     )
+from .errors import OperationAborted
 from .glicko2 import Glicko2, DRAW, LOSS, WIN
+from .utils import normalize
 
 
 logger = logging.getLogger(__name__)
@@ -52,40 +55,40 @@ class Rating(GloballyUnique, Base):
         intid_t, Sequence('gen_idrating', optional=True),
         primary_key=True,
         nullable=False,
-        info=dict(label=N_('Rating ID'),
-                  hint=N_('Unique ID of the rating.')))
+        info=dict(label=_('Rating ID'),
+                  hint=_('Unique ID of the rating.')))
     """Primary key."""
 
     idowner = Column(
-        intid_t, ForeignKey('players.idplayer', ondelete="SET NULL"),
-        info=dict(label=N_('Owner ID'),
-                  hint=N_('ID of the user that is responsible for this record.')))
-    """ID of the :py:class:`user <.Player>` that is responsible for this record."""
+        intid_t, ForeignKey('users.iduser', name='fk_rating_owner', ondelete="SET NULL"),
+        info=dict(label=_('Owner ID'),
+                  hint=_('ID of the user that is responsible for this record.')))
+    """ID of the :py:class:`user <.User>` that is responsible for this record."""
 
     idclub = Column(
-        intid_t, ForeignKey('clubs.idclub'),
-        info=dict(label=N_('Club ID'),
-                  hint=N_('ID of the club the rating is restricted to.')))
+        intid_t, ForeignKey('clubs.idclub', name='fk_rating_club', ondelete="SET NULL"),
+        info=dict(label=_('Club ID'),
+                  hint=_('ID of the club the rating is restricted to.')))
     """Restricted to :py:class:`club <.Club>`'s ID."""
 
     description = Column(
         description_t,
         nullable=False,
-        info=dict(label=N_('Rating'),
-                  hint=N_('Description of the rating.')))
+        info=dict(label=_('Description'),
+                  hint=_('Description of the rating.')))
     """Description of the rating."""
 
     level = Column(
         flag_t,
         nullable=False,
-        info=dict(label=N_('Level'),
-                  hint=N_('Rating level.'),
-                  dictionary=dict((str(i),v) for i,v in enumerate((
-                      N_('Historical (imported) rating'),
-                      N_('Level 1, international tourneys'),
-                      N_('Level 2, national/open tourneys'),
-                      N_('Level 3, regional tourneys'),
-                      N_('Level 4, courtyard tourneys'),
+        info=dict(label=_('Level'),
+                  hint=_('Rating level.'),
+                  dictionary=dict((str(i), v) for i, v in enumerate((
+                      _('Historical (imported) rating'),
+                      _('Level 1, international tourneys'),
+                      _('Level 2, national/open tourneys'),
+                      _('Level 3, regional tourneys'),
+                      _('Level 4, courtyard tourneys'),
                   )))))
     """Rating level."""
 
@@ -93,8 +96,8 @@ class Rating(GloballyUnique, Base):
         prize_t,
         nullable=False,
         default='0.5',
-        info=dict(label=N_('Tau'),
-                  hint=N_('The TAU value for the Glicko2 algorithm.'),
+        info=dict(label=_('Tau'),
+                  hint=_('The TAU value for the Glicko2 algorithm.'),
                   min=0.01, max=2))
     """Value of TAU for the Glicko2 algorithm."""
 
@@ -102,8 +105,8 @@ class Rating(GloballyUnique, Base):
         smallint_t,
         nullable=False,
         default=1500,
-        info=dict(label=N_('Rate'),
-                  hint=N_('The default rate value for the Glicko2 algorithm.'),
+        info=dict(label=_('Rate'),
+                  hint=_('The default rate value for the Glicko2 algorithm.'),
                   min=1, max=3000))
     """Default value of rate (MU) for the Glicko2 algorithm."""
 
@@ -111,8 +114,8 @@ class Rating(GloballyUnique, Base):
         smallint_t,
         nullable=False,
         default=350,
-        info=dict(label=N_('Deviation'),
-                  hint=N_('The default deviation value for the Glicko2 algorithm.'),
+        info=dict(label=_('Deviation'),
+                  hint=_('The default deviation value for the Glicko2 algorithm.'),
                   min=1, max=500))
     """Default value of deviation (PHI) for the Glicko2 algorithm."""
 
@@ -120,8 +123,8 @@ class Rating(GloballyUnique, Base):
         volatility_t,
         nullable=False,
         default='0.06',
-        info=dict(label=N_('Volatility'),
-                  hint=N_('The default volatility value for the Glicko2 algorithm.'),
+        info=dict(label=_('Volatility'),
+                  hint=_('The default volatility value for the Glicko2 algorithm.'),
                   min=0.00001, max=1))
     """Default value of volatility (SIGMA) for the Glicko2 algorithm."""
 
@@ -129,17 +132,17 @@ class Rating(GloballyUnique, Base):
         boolean_t,
         nullable=False,
         default=False,
-        info=dict(label=N_('Inherit'),
-                  hint=N_('Whether to lookup rates in higher levels ratings.')))
+        info=dict(label=_('Inherit'),
+                  hint=_('Whether to lookup rates in higher levels ratings.')))
     """Whether to lookup rates in higher levels ratings."""
 
     lower_rate = Column(
         smallint_t,
         nullable=False,
         default=1600,
-        info=dict(label=N_('Lower points'),
-                  hint=N_('Lower value of the range used to interpolate players rates'
-                          ' when (almost) all competitors are unrated.')))
+        info=dict(label=_('Lower points'),
+                  hint=_('Lower value of the range used to interpolate players rates'
+                         ' when (almost) all competitors are unrated.')))
     """
     Lower value of the range used to interpolate players rates when (almost) all competitors
     are unrated.
@@ -149,20 +152,20 @@ class Rating(GloballyUnique, Base):
         smallint_t,
         nullable=False,
         default=2600,
-        info=dict(label=N_('Higher points'),
-                  hint=N_('Higher value of the range used to interpolate players rates.')))
+        info=dict(label=_('Higher points'),
+                  hint=_('Higher value of the range used to interpolate players rates.')))
     """Higher value of the range used to interpolate players rates."""
 
     outcomes = Column(
         code_t,
         nullable=False,
         default='guido',
-        info=dict(label=N_('Match outcomes'),
-                  hint=N_('Kind of formula used to compute match outcomes.'),
+        info=dict(label=_('Match outcomes'),
+                  hint=_('Kind of formula used to compute match outcomes.'),
                   dictionary=dict(
-                      glicko=N_('Standard Glicko'),
-                      guido=N_("Guido's formula"),
-                      expds=N_('Exponential on scores difference'))))
+                      glicko=_('Standard Glicko'),
+                      guido=_("Guido's formula"),
+                      expds=_('Exponential on scores difference'))))
     """Kind of formula used to compute match outcomes.
 
     This is used to determine which formula will be used to compute the match outcomes to feed
@@ -182,21 +185,67 @@ class Rating(GloballyUnique, Base):
 
     ## Relations
 
-    owner = relationship('Player', backref='owned_ratings')
-    """The :py:class:`owner <.Player>` of this record, `admin` when ``None``."""
+    owner = relationship('User', backref='owned_ratings')
+    """The :py:class:`owner <.User>` of this record, `admin` when ``None``."""
+
+    championships = relationship('Championship', backref='rating',
+                                 passive_updates=False)
+    """:py:class:`Championships <.Championship>` using this rating by default."""
 
     tourneys = relationship('Tourney', backref='rating',
                             passive_updates=False,
-                            order_by="Tourney.date",
-                            lazy=True)
+                            order_by="Tourney.date")
     """:py:class:`Tourneys <.Tourney>` using this rating."""
 
     rates = relationship('Rate', backref='rating',
                          cascade="all, delete-orphan",
-                         order_by="Rate.date, Rate.idplayer",
-                         lazy=True)
+                         order_by="Rate.date, Rate.idplayer")
     """List of :py:class:`rates <.Rate>`."""
 
+    @classmethod
+    def check_insert(klass, session, fields, user_id):
+        "Check description validity"
+
+        from .club import Club, clubusers
+
+        try:
+            desc = normalize(fields['description'])
+        except KeyError:
+            raise OperationAborted(_('For a new rating the "description" field'
+                                     ' is mandatory'))
+
+        if not desc:
+            raise OperationAborted(_('For a new rating the "description" field'
+                                     ' is mandatory'))
+
+        if user_id != 'admin':
+            idclub = fields.get('idclub')
+
+            if idclub is None:
+                raise OperationAborted(_('You are not allowed to add a global rating,'
+                                         ' not associated to a specific club'))
+
+            clubs = Club.__table__
+            if not session.scalar(select([True])
+                                  .where(or_(
+                                      exists()
+                                      .where(clubs.c.idclub == idclub)
+                                      .where(clubs.c.idowner == user_id),
+                                      exists()
+                                      .where(clubusers.c.idclub == idclub)
+                                      .where(clubusers.c.iduser == user_id)))):
+                raise OperationAborted(_('You are not allowed to add a rating to the'
+                                         ' selected club'))
+
+    def check_update(self, fields):
+        "Check description validity"
+
+        if 'description' in fields:
+            desc = normalize(fields['description'])
+            if not desc:
+                raise OperationAborted(_('The "description" field of a rating'
+                                         ' cannot be empty'))
+
     def getPlayerRating(self, player, before=None):
         """Return the rate of a `player`
 
@@ -286,8 +335,8 @@ class Rating(GloballyUnique, Base):
 
         rt = Rate.__table__
 
-        timespan = select([func.min(rt.c.date), func.max(rt.c.date)]) \
-                   .where(rt.c.idrating == self.idrating)
+        timespan = (select([func.min(rt.c.date), func.max(rt.c.date)])
+                    .where(rt.c.idrating == self.idrating))
         return s.execute(timespan).first()
 
     def isPhantom(self, competitor):
@@ -302,7 +351,7 @@ class Rating(GloballyUnique, Base):
         return (competitor is None
                 or (competitor.points == 0
                     and competitor.totscore == 0
-                    and competitor.netscore % 25 == 0))
+                    and competitor.netscore % competitor.tourney.phantomscore == 0))
 
     def _computeGlickoOutcomes(self, score1, score2):
         # Standard Glicko, best suited to Chess matches
@@ -379,7 +428,7 @@ class Rating(GloballyUnique, Base):
 
         try:
             compute_outcomes = getattr(self, "_compute%sOutcomes" % self.outcomes.capitalize())
-        except AttributeError:
+        except AttributeError:  # pragma: nocover
             raise AttributeError("No %r method to compute match outcomes" % self.outcomes)
 
         logger.debug('Using the %r method to compute match outcomes', self.outcomes)
@@ -431,6 +480,9 @@ class Rating(GloballyUnique, Base):
             if not tourney.prized:
                 continue
 
+            if tourney.system == 'knockout':
+                continue
+
             logger.debug('Considering tourney %s', tourney)
 
             outcomes = defaultdict(list)
@@ -457,7 +509,7 @@ class Rating(GloballyUnique, Base):
                 if c2.idplayer1 not in rcache:
                     rcache[c2.idplayer1] = self.getPlayerRating(c2.player1,
                                                                 tourney.date)
-                    if logger.isEnabledFor(logging.DEBUG):
+                    if logger.isEnabledFor(logging.DEBUG):  # pragma: nocover
                         logger.debug('%s rate is: %s',
                                      s.query(Player).get(c2.idplayer1),
                                      rcache[c2.idplayer1])
@@ -468,7 +520,7 @@ class Rating(GloballyUnique, Base):
                 if c1.idplayer1 not in rcache:
                     rcache[c1.idplayer1] = self.getPlayerRating(c1.player1,
                                                                 tourney.date)
-                    if logger.isEnabledFor(logging.DEBUG):
+                    if logger.isEnabledFor(logging.DEBUG):  # pragma: nocover
                         logger.debug('%s rate is: %s',
                                      s.query(Player).get(c1.idplayer1),
                                      rcache[c1.idplayer1])
@@ -483,7 +535,7 @@ class Rating(GloballyUnique, Base):
             for idplayer in outcomes:
                 current = rcache[idplayer]
 
-                if logger.isEnabledFor(logging.DEBUG):
+                if logger.isEnabledFor(logging.DEBUG):  # pragma: nocover
                     logger.debug('Computing new rate for %s', s.query(Player).get(idplayer))
                     logger.debug('Player current rate: %s', current)
                     logger.debug('Player outcomes: %s', outcomes[idplayer])
@@ -512,8 +564,9 @@ class Rating(GloballyUnique, Base):
 
     def update(self, data, missing_only=False):
         for field in ('tau', 'default_volatility'):
-            if field in data and isinstance(data[field], str):
-                data[field] = Decimal(data[field])
+            value = data.get(field, None)
+            if isinstance(value, str):
+                data[field] = Decimal(value)
         return super().update(data, missing_only)
 
     def serialize(self, serializer):
@@ -528,7 +581,7 @@ class Rating(GloballyUnique, Base):
         simple['guid'] = self.guid
         simple['modified'] = self.modified
         if self.idowner:
-            simple['owner'] = serializer.addPlayer(self.owner)
+            simple['owner'] = serializer.addUser(self.owner)
         if self.idclub:
             simple['club'] = serializer.addClub(self.club)
         simple['description'] = self.description
@@ -553,7 +606,7 @@ def interpolate_unrated(cache, ranking, glicko2, phantom_p, lower_rate, higher_r
     sumx = sumy = sumxy = sumx2 = phantoms = 0
 
     for x, competitor in enumerate(ranking, 1):
-        # Do not consider retired players when their only match was against the phantom
+        # Do not consider withdrawn players when their only match was against the phantom
         if phantom_p(competitor) or competitor.idplayer1 not in cache:
             phantoms += 1
             continue
