@@ -1,8 +1,9 @@
 // -*- coding: utf-8 -*-
-// :Progetto:  SoL -- Tourney competitors management
-// :Creato:    gio 02 mag 2013 09:54:44 CEST
-// :Autore:    Lele Gaifax <lele@metapensiero.it>
-// :Licenza:   GNU General Public License version 3 or later
+// :Project:   SoL -- Tourney competitors management
+// :Created:   gio 02 mag 2013 09:54:44 CEST
+// :Author:    Lele Gaifax <lele@metapensiero.it>
+// :License:   GNU General Public License version 3 or later
+// :Copyright: © 2013, 2014, 2018, 2020, 2021 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -61,7 +62,7 @@ Ext.define('SoL.module.Competitors', {
                     '*': {
                         editor: function(finfo) {
                             var c = {};
-                            if(finfo.name.substr(7) !== 'FullName')
+                            if(finfo.name.substr(7) !== 'FullName' && finfo.name !== 'position')
                                 c.readonly = true;
                             return c;
                         }
@@ -88,10 +89,10 @@ Ext.define('SoL.module.Competitors', {
     },
 
     createOrShowWindow: function(idtourney, tourney, date, championship, playersperteam) {
-        var me = this;
-        var config = me.config;
-        var desktop = me.app.getDesktop();
-        var win = desktop.getWindow(me.id);
+        var me = this,
+            config = me.config,
+            desktop = me.app.getDesktop(),
+            win = desktop.getWindow(me.id);
 
         // If the window is already present, destroy and recreate it,
         // to reapply configuration and filters
@@ -107,7 +108,8 @@ Ext.define('SoL.module.Competitors', {
                 config = Ext.apply({
                     stickyFilters: [{
                         property: 'idtourney',
-                        value: idtourney
+                        value: idtourney,
+                        operator: '='
                     }]
                 }, config);
 
@@ -133,8 +135,9 @@ Ext.define('SoL.module.Competitors', {
                             var wh = Ext.create('SoL.window.Help', {
                                 width: whsize.width,
                                 height: whsize.height,
-                                // TRANSLATORS: this is the URL of the manual
-                                // page explaining players management
+                                // TRANSLATORS: this is the URL of the manual page explaining
+                                // competitors management, do not change unless the manual is
+                                // actually translated in the target language
                                 help_url: _('/static/manual/en/competitors.html'),
                                 title: _('Help on competitors management')
                             });
