@@ -1,8 +1,9 @@
 // -*- coding: utf-8 -*-
-// :Progetto:  SoL -- Players window
-// :Creato:    ven 17 ott 2008 23:39:25 CEST
-// :Autore:    Lele Gaifax <lele@metapensiero.it>
-// :Licenza:   GNU General Public License version 3 or later
+// :Project:   SoL -- Players window
+// :Created:   ven 17 ott 2008 23:39:25 CEST
+// :Author:    Lele Gaifax <lele@metapensiero.it>
+// :License:   GNU General Public License version 3 or later
+// :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -13,140 +14,6 @@
 /*jsl:declare SoL*/
 /*jsl:declare FileReader*/
 
-Ext.define('SoL.module.Players.AssignOwnership', {
-    extend: 'MP.action.StoreAware',
-    uses: [
-        'Ext.Action',
-        'MP.form.Panel',
-        'MP.window.Notification'
-    ],
-
-    statics: {
-        ASSIGN_OWNERSHIP_ACTION: 'assign_ownership'
-    },
-
-    initActions: function() {
-        var me = this;
-        var ids = me.statics();
-
-        me.callParent();
-
-        me.assignOwnershipAction = me.addAction(new Ext.Action({
-            itemId: ids.ASSIGN_OWNERSHIP_ACTION,
-            text: _('Assign'),
-            tooltip: _('Assign ownership of selected records.'),
-            iconCls: 'owner-icon',
-            disabled: true,
-            needsSelectedRow: true,
-            handler: function() {
-                me.assignOwnership();
-            }
-        }));
-    },
-
-    attachActions: function() {
-        var me = this;
-
-        me.callParent();
-
-        if(me.module.app.user.is_admin || me.module.app.user.is_ownersadmin) {
-            var tbar = me.component.child('#ttoolbar');
-
-            tbar.add(tbar.items.length-3, me.assignOwnershipAction);
-        }
-    },
-
-    assignOwnership: function() {
-        var me = this;
-        var desktop = me.module.app.getDesktop();
-        var win;
-        var winWidth = 345;
-        var winHeight = 110;
-
-        var model = Ext.define('MP.data.ImplicitModel-'+Ext.id(), {
-            extend: 'Ext.data.Model',
-            fields: ['idplayer', 'Fullname'],
-            idProperty: 'idplayer'
-        });
-
-        var lstore = Ext.create('Ext.data.Store', {
-            model: model,
-            pageSize: 999,
-            autoLoad: false,
-            remoteFilter: true,
-            proxy: {
-                type: 'ajax',
-                url: '/data/owners',
-                filterParam: 'filters',
-                reader: {
-                    type: 'json',
-                    root: 'root',
-                    idProperty: 'idplayer'
-                }
-            }
-        });
-        lstore.implicitModel = true;
-
-        var form = new Ext.form.Panel({
-            frame: true,
-            bodyPadding: '10 10 0',
-            defaults: {
-                labelWidth: 90,
-                anchor: '100%'
-            },
-            items: [{
-                xtype: 'combo',
-                store: lstore,
-                valueField: 'idplayer',
-                displayField: 'Fullname',
-                buttonText: _('Browse'),
-                emptyText: _('Select responsible'),
-                allowBlank: false,
-                triggerAction: 'all',
-                typeAhead: true,
-                forceSelection: true,
-                fieldLabel: _('Owner')
-            }],
-            buttons: [{
-                text: _('Cancel'),
-                handler: function() {
-                    win.destroy();
-                }
-            }, {
-                text: _('Confirm'),
-                formBind: true,
-                handler: function() {
-                    var frm = form.getForm();
-                    if(frm.isValid()) {
-                        var combo = frm.getFields().getAt(0);
-                        var sels = me.component.getSelectionModel().getSelection();
-                        Ext.each(sels, function(record) {
-                            record.set('idowner', combo.value);
-                            record.set('Owner', combo.getRawValue());
-                        });
-                        win.destroy();
-                    }
-                }
-            }]
-        });
-
-        win = desktop.createWindow({
-            id: me.id,
-            title: _('Assign ownership'),
-            iconCls: 'upload-icon',
-            width: winWidth,
-            height: winHeight,
-            layout: 'fit',
-            minimizable: false,
-            maximizable: false,
-            modal: true,
-            items: [form]
-        });
-
-        win.show();
-    }
-});
-
 Ext.define('SoL.module.Players.Actions', {
     extend: 'MP.action.StoreAware',
     uses: [
@@ -160,17 +27,20 @@ Ext.define('SoL.module.Players.Actions', {
 
     statics: {
         EDIT_PLAYER_ACTION: 'edit_player',
+        SHOW_TOURNEYS_ACTION: 'show_tourneys',
         SHOW_DUPLICATED_ACTION: 'show_duplicates',
-        SHOW_DISTRIBUTION_ACTION: 'show_distribution'
+        SHOW_DISTRIBUTION_ACTION: 'show_distribution',
+        SHOW_LIT_PAGE_ACTION: 'show_lit',
+        SET_AGREED_PRIVACY_ACTION: 'set_agreed_privacy'
     },
 
     initActions: function() {
-        var me = this;
-        var ids = me.statics();
+        var me = this,
+            ids = me.statics();
 
         me.callParent();
 
-        me.editPlayerAction = me.addAction(new Ext.Action({
+        me.editPlayer = me.addAction(new Ext.Action({
             itemId: ids.EDIT_PLAYER_ACTION,
             text: _('Modify'),
             tooltip: _('Edit selected player.'),
@@ -183,15 +53,15 @@ Ext.define('SoL.module.Players.Actions', {
             }
         }));
 
-        me.showDuplicatedPlayersAction = me.addAction(new Ext.Action({
+        me.showDuplicatedPlayers = me.addAction(new Ext.Action({
             itemId: ids.SHOW_DUPLICATED_ACTION,
             text: _('Duplicates'),
             tooltip: _('Show possibly duplicated players that should be merged together.'),
             iconCls: 'dup-users-icon',
             needsCleanStore: true,
             handler: function() {
-                var store = me.component.store;
-                var win = me.component.up();
+                var store = me.component.store,
+                    win = me.component.up();
 
                 if(this.dups) {
                     store.proxy.url = me.component.dataURL;
@@ -213,7 +83,7 @@ Ext.define('SoL.module.Players.Actions', {
             }
         }));
 
-        me.showTourneysAction = me.addAction(new Ext.Action({
+        me.showTourneys = me.addAction(new Ext.Action({
             itemId: ids.SHOW_TOURNEYS_ACTION,
             text: _('Tourneys'),
             tooltip: _('Show tourneys played by the selected player.'),
@@ -222,48 +92,113 @@ Ext.define('SoL.module.Players.Actions', {
             needsOneSelectedRow: true,
             needsCleanStore: true,
             handler: function() {
-                var record = me.component.getSelectionModel().getSelection()[0];
-                var idplayer = record.get('idplayer');
-                var player = SoL.module.Players.full_name(record);
-                var module = me.module.app.getModule('tourneys-win');
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    idplayer = record.get('idplayer'),
+                    player = SoL.module.Players.full_name(record),
+                    module = me.module.app.getModule('tourneys-win');
                 module.createOrShowWindow('players', idplayer, player);
             }
         }));
 
-        me.showDistributionAction = me.addAction(new Ext.Action({
+        me.showDistribution = me.addAction(new Ext.Action({
             itemId: ids.SHOW_DISTRIBUTION_ACTION,
             text: _('Distribution'),
             tooltip: _('Show players distribution around the globe.'),
             iconCls: 'players-distribution-icon',
             handler: function() {
-                var desktop = me.module.app.getDesktop();
-                var url = '/svg/playersdist?' + Ext.Object.toQueryString({
-                    width: Math.floor(desktop.getWidth() * 0.8),
-                    height: Math.floor(desktop.getHeight() * 0.8)
-                });
+                var desktop = me.module.app.getDesktop(),
+                    url = '/svg/playersdist?' + Ext.Object.toQueryString({
+                        width: Math.floor(desktop.getWidth() * 0.8),
+                        height: Math.floor(desktop.getHeight() * 0.8)
+                    });
 
                 window.open(url, "_blank");
             }
         }));
+
+        me.showLitPage = me.addAction(new Ext.Action({
+            itemId: ids.SHOW_LIT_PAGE_ACTION,
+            text: _('Lit page'),
+            tooltip: _('Show the corresponding Lit page.'),
+            iconCls: 'lit-icon',
+            disabled: true,
+            needsOneSelectedRow: true,
+            handler: function() {
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    guid = record.get('guid'),
+                    url = '/lit/player/' + guid;
+                window.open(url, "_blank");
+            }
+        }));
+
+        me.setAgreedPrivacy = me.addAction(new Ext.Action({
+            itemId: ids.SET_AGREED_PRIVACY_ACTION,
+            text: _('Privacy'),
+            tooltip: _('Make the selected players “discernible” in public views.'),
+            iconCls: 'agreed-privacy-icon',
+            disabled: true,
+            needsSelectedRow: true,
+            handler: function() {
+                var sels = me.component.getSelectionModel().getSelection(),
+                    currentuser = me.module.app.user,
+                    all_owned = true;
+                if(!currentuser.is_admin) {
+                    Ext.each(sels, function(record) {
+                        if(record.get('idowner') !== null
+                           && record.get('idowner') !== currentuser.user_id) {
+                            all_owned = false;
+                            return false;
+                        }
+                        return true;
+                    });
+                }
+                if(all_owned) {
+                    Ext.each(sels, function(record) {
+                        record.set('agreedprivacy', 'A');
+                    });
+                    Ext.create("MP.window.Notification", {
+                        position: 't',
+                        width: 260,
+                        title: _('Changes have been applied…'),
+                        html: _('Your changes have been applied <strong>locally</strong>.<br/><br/>To make them permanent you must click on the <blink>Save</blink> button.'),
+                        iconCls: 'info-icon'
+                    }).show();
+                } else {
+                    Ext.create("MP.window.Notification", {
+                        position: 'br',
+                        width: 260,
+                        title: _('Unauthorized'),
+                        html: _('At least one of the selected players does not belong to you, and thus cannot be modified, sorry!'),
+                        iconCls: 'alert-icon'
+                    }).show();
+                }
+            }
+        }));
     },
 
     attachActions: function() {
-        var me = this;
+        var me = this,
+            currentuser = me.module.app.user;
 
         me.callParent();
 
         var tbar = me.component.child('#ttoolbar');
 
-        tbar.add(2, ' ',
-                 me.editPlayerAction,
-                 me.showTourneysAction,
-                 me.showDuplicatedPlayersAction,
-                 me.showDistributionAction);
+        if(currentuser.is_playersmanager)
+            tbar.add(2, ' ',
+                     me.editPlayer,
+                     me.showTourneys,
+                     me.showDuplicatedPlayers,
+                     me.showDistribution,
+                     me.setAgreedPrivacy);
+        else
+            tbar.add(0, me.showTourneys,
+                     me.showDistribution);
 
         me.component.on({
             itemdblclick: function() {
-                if(!me.editPlayerAction.isDisabled())
-                    me.editPlayerAction.execute();
+                if(!me.editPlayer.isDisabled())
+                    me.editPlayer.execute();
             }
         });
         me.component.store.on({
@@ -276,17 +211,22 @@ Ext.define('SoL.module.Players.Actions', {
     },
 
     shouldDisableAction: function(act) {
-        var me = this;
-        var disable = me.component.shouldDisableAction(act);
-        var statics = me.statics();
-        var currentuser = me.module.app.user;
+        var me = this,
+            disable = me.component.shouldDisableAction(act),
+            statics = me.statics(),
+            currentuser = me.module.app.user;
 
         if(!disable && !currentuser.is_admin) {
             if(act.itemId == statics.EDIT_PLAYER_ACTION) {
-                var record = me.component.getSelectionModel().getSelection()[0];
-
-                if(record.get('idowner') != currentuser.user_id)
+                if(!me.module.app.user.is_playersmanager)
                     disable = true;
+                else {
+                    var record = me.component.getSelectionModel().getSelection()[0];
+
+                    if(record.get('idowner') !== null
+                       && record.get('idowner') !== currentuser.user_id)
+                        disable = true;
+                }
             }
         }
 
@@ -303,9 +243,11 @@ Ext.define('SoL.module.Players.Actions', {
 
             reader.onload = function(e) {
                 var img = e.target.result;
-                if(img.length > 256000) {
+                // Multiply by 1.33, because here we have a data URI, in
+                // base 64
+                if(img.length > (512000 * 1.33)) {
                     Ext.MessageBox.alert(_('Error'),
-                                         _('Image too big, max 256k allowed'));
+                                         _('Image too big, max 512k allowed'));
                     form._portrait = null;
                 } else {
                     form.down('image').setSrc(img);
@@ -332,9 +274,9 @@ Ext.define('SoL.module.Players.Actions', {
     },
 
     showEditPlayerWindow: function(record) {
-        var me = this;
-        var desktop = me.module.app.getDesktop();
-        var win = desktop.getWindow('edit-player-win');
+        var me = this,
+            desktop = me.module.app.getDesktop(),
+            win = desktop.getWindow('edit-player-win');
 
         // If the window is already present, destroy and recreate it,
         // to reapply configuration and filters
@@ -342,25 +284,29 @@ Ext.define('SoL.module.Players.Actions', {
             win.destroy();
         }
 
-        var metadata = me.module.config.metadata;
-        var size = desktop.getReasonableWindowSize(1000, 480);
-        var editors = metadata.editors({
-            '*': {
-                editor: MP.form.Panel.getDefaultEditorSettingsFunction('100%')
-            },
-            nationality: { editor: { xtype: 'flagscombo' } },
-            Language: { editor: { queryMode: 'local' } }
-        });
-        var secondcoleditors = [
-            editors.Language,
-            editors.nationality,
-            editors.citizenship,
-            editors.Club,
-            editors.Federation,
-            editors.Owner
-        ];
+        var metadata = me.module.config.metadata,
+            size = desktop.getReasonableWindowSize(1000, 420),
+            editors = metadata.editors({
+                '*': {
+                    editor: MP.form.Panel.getDefaultEditorSettingsFunction('100%')
+                },
+                firstname: { editor: { minLength: 2 } },
+                lastname: { editor: { minLength: 2 } },
+                nationality: { editor: { xtype: 'flagscombo' } },
+                Language: { editor: { queryMode: 'local' } }
+            }),
+            secondcoleditors = [
+                editors.Language,
+                editors.nationality,
+                editors.citizenship,
+                editors.Club,
+                editors.Federation,
+                editors.Owner
+            ];
+
         if(me.module.app.user.is_admin || me.module.app.user.is_ownersadmin)
             secondcoleditors.push(editors.ownersadmin);
+
         var form = Ext.create('MP.form.Panel', {
             autoScroll: true,
             fieldDefaults: {
@@ -379,11 +325,10 @@ Ext.define('SoL.module.Players.Actions', {
                         editors.firstname,
                         editors.lastname,
                         editors.nickname,
-                        editors.password,
                         editors.sex,
                         editors.birthdate,
                         editors.email,
-                        editors.phone
+                        editors.agreedprivacy
                     ]
                 }, {
                     xtype: 'container',
@@ -492,8 +437,9 @@ Ext.define('SoL.module.Players.Actions', {
                     var wh = Ext.create('SoL.window.Help', {
                         width: whsize.width,
                         height: whsize.height,
-                        // TRANSLATORS: this is the URL of the manual
-                        // page explaining championship insert/edit
+                        // TRANSLATORS: this is the URL of the manual page explaining players
+                        // insert/edit, do not change unless the manual is actually translated
+                        // in the target language
                         help_url: _('/static/manual/en/players.html#insert-and-edit'),
                         title: _('Help on player insert/edit')
                     });
@@ -526,26 +472,27 @@ Ext.define('SoL.module.Players', {
         'Ext.grid.plugin.DragDrop',
         'SoL.form.field.FlagsCombo',
         'SoL.module.Players.Actions',
-        'SoL.module.Players.AssignOwnership',
         'SoL.module.Players.MergePlugin',
+        'SoL.module.Users',
         'SoL.window.Help'
     ],
 
     statics: {
         full_name: function(player, html) {
-            var lastname = player.get('lastname');
-            var firstname = player.get('firstname');
-            var nickname = player.get('nickname');
-            var format = function(fmt, data) {
-                return fmt.replace(/\{(\w+)\}/g, function(m, i) {
-                    return data[i];
-                });
-            };
+            var lastname = player.get('lastname'),
+                firstname = player.get('firstname'),
+                nickname = player.get('nickname'),
+                format = function(fmt, data) {
+                    return fmt.replace(/\{(\w+)\}/g, function(m, i) {
+                        return data[i];
+                    });
+                };
+
+            if(nickname) {
+                var nnlower = nickname.toLowerCase(),
+                    fnlower = firstname.toLowerCase(),
+                    lnlower = lastname.toLowerCase();
 
-            if(nickname !== '') {
-                var nnlower = nickname.toLowerCase();
-                var fnlower = firstname.toLowerCase();
-                var lnlower = lastname.toLowerCase();
                 if(nnlower != fnlower && nnlower != lnlower) {
                     nickname = format('“{nickname}”', {
                         nickname: nickname
@@ -648,7 +595,7 @@ Ext.define('SoL.module.Players', {
                     plugins: [
                         Ext.create('SoL.module.Players.Actions',
                                    { module: me }),
-                        Ext.create('SoL.module.Players.AssignOwnership',
+                        Ext.create('SoL.module.Users.AssignOwnership',
                                    { module: me })
                     ]
                 });
@@ -660,11 +607,12 @@ Ext.define('SoL.module.Players', {
         }
     },
 
-    createOrShowWindow: function(filter_ids, buttons, idclub, club, isfederation) {
-        var me = this;
-        var config = me.config;
-        var desktop = me.app.getDesktop();
-        var win = desktop.getWindow(me.id);
+    createOrShowWindow: function(caller, id, description, isfederation) {
+        var me = this,
+            config = me.config,
+            desktop = me.app.getDesktop(),
+            win = desktop.getWindow(me.id),
+            currentuser = me.app.user;
 
         // If the window is already present, destroy and recreate it,
         // to reapply configuration and filters
@@ -675,8 +623,8 @@ Ext.define('SoL.module.Players', {
         me.configure(
             [me.getConfig],
             function(done) {
-                var size = desktop.getReasonableWindowSize(690, 640, "SE");
-                var filteredOn;
+                var size = desktop.getReasonableWindowSize(800, 640, "SE"),
+                    filteredOn = '';
 
                 config = Ext.apply({
                     selModel: {
@@ -699,57 +647,75 @@ Ext.define('SoL.module.Players', {
                     }
                 }, config);
 
-                if(filter_ids && filter_ids.length>0) {
-                    config.stickyFilters = [{
-                        id: 'ids',
-                        property: "idplayer",
-                        value: filter_ids.join(','),
-                        operator: '<>'
-                    }];
-                }
+                if(!currentuser.is_playersmanager)
+                    config.noAddAndDelete = true;
 
-                if(buttons) {
-                    if(buttons.isPlugin) {
-                        config.plugins = [buttons].concat(config.plugins);
-                    } else {
-                        config.buttons = buttons;
-                    }
-                }
-
-                if(club) {
+                if(caller == 'clubs') {
                     if(isfederation) {
                         filteredOn = ' (' + Ext.String.format(
-                            // TRANSLATORS: this is the explanation on the players
-                            // window title when the filter is a federation
-                            _('associated with {0}'), club) + ')';
+                            // TRANSLATORS: this is the explanation on the window title when
+                            // the filter is a federation
+                            _('associated with {0}'), description) + ')';
                         Ext.apply(config, {
                             newRecordData: {
-                                idfederation: idclub,
-                                Federation: club
+                                idfederation: id,
+                                Federation: description
                             },
                             stickyFilters: [{
                                 property: 'idfederation',
-                                value: idclub
+                                value: id,
+                                operator: '='
                             }]
                         });
                     } else {
                         filteredOn = ' (' + Ext.String.format(
-                            // TRANSLATORS: this is the explanation on the players
-                            // window title when the filter is a club
-                            _('members of {0}'), club) + ')';
+                            // TRANSLATORS: this is the explanation on the window title when
+                            // the filter is a club
+                            _('members of {0}'), description) + ')';
                         Ext.apply(config, {
                             newRecordData: {
-                                idclub: idclub,
-                                Club: club
+                                idclub: id,
+                                Club: description
                             },
                             stickyFilters: [{
                                 property: 'idclub',
-                                value: idclub
+                                value: id,
+                                operator: '='
                             }]
                         });
                     }
-                } else {
-                    filteredOn = '';
+                } else if(caller == 'competitors') {
+                    // id is actually an array of players IDs...
+                    if(id && id.length > 0) {
+                        config.stickyFilters = [{
+                            id: 'ids',
+                            property: "idplayer",
+                            value: id.join(','),
+                            operator: '<>'
+                        }];
+                    }
+
+                    // ... and description is either an Actions mixin or an array of additional
+                    // buttons
+                    if(description) {
+                        if(description.isPlugin) {
+                            config.plugins = [description].concat(config.plugins);
+                        } else {
+                            config.buttons = description;
+                        }
+                    }
+                } else if(caller == 'users') {
+                    filteredOn = ' (' + Ext.String.format(
+                        // TRANSLATORS: this is the explanation on the window title when the
+                        // filter is a user
+                        _('owned by {0}'), description) + ')';
+                    config = Ext.apply({
+                        filters: [{
+                            property: 'idowner',
+                            value: id,
+                            operator: '='
+                        }]
+                    }, config);
                 }
 
                 win = desktop.createWindow({
@@ -770,8 +736,9 @@ Ext.define('SoL.module.Players', {
                             var wh = Ext.create('SoL.window.Help', {
                                 width: whsize.width,
                                 height: whsize.height,
-                                // TRANSLATORS: this is the URL of the manual
-                                // page explaining players management
+                                // TRANSLATORS: this is the URL of the manual page explaining
+                                // players management, do not change unless the manual is
+                                // actually translated in the target language
                                 help_url: _('/static/manual/en/players.html'),
                                 title: _('Help on players management')
                             });
@@ -792,25 +759,29 @@ Ext.define('SoL.module.Players', {
                     }
                 });
 
-                var da = grid.findActionById('delete');
-                da.shouldBeDisabled = me.shouldDisableDeleteAction.bind(grid);
+                if(currentuser.is_playersmanager) {
+                    var da = grid.findActionById('delete');
+                    da.shouldBeDisabled = me.shouldDisableDeleteAction.bind(grid);
+                }
             }
         );
     },
 
     shouldDisableDeleteAction: function() {
-        var grid = this;
-        var sm = grid.getSelectionModel();
-        var currentuser = grid.up().up().app.user;
+        var grid = this,
+            sm = grid.getSelectionModel(),
+            currentuser = grid.up().up().app.user;
 
         if(sm.getCount() > 0) {
-            var selrecs = sm.getSelection();
-            var disable = false;
+            var selrecs = sm.getSelection(),
+                disable = false;
 
             for(var i=selrecs.length-1; i>=0; i--) {
                 var record = selrecs[i];
 
-                if(!currentuser.is_admin && currentuser.user_id != record.get('idowner')) {
+                if(!currentuser.is_admin
+                   && record.get('idowner') !== null
+                   && currentuser.user_id !== record.get('idowner')) {
                     disable = true;
                     break;
                 }
@@ -890,8 +861,9 @@ Ext.define('SoL.module.Players.MergeTarget', {
             return false;
         }
 
-        var app = me.view.up().up().up().app;
-        var currentuser = app.user;
+        var app = me.view.up().up().up().app,
+            currentuser = app.user;
+
         if(!currentuser.is_admin) {
             for(var i=data.records.length-1; i>=0; i--) {
                 if(currentuser.user_id != data.records[i].get('idowner')) {
@@ -900,9 +872,10 @@ Ext.define('SoL.module.Players.MergeTarget', {
             }
         }
 
-        var trec = me.view.getRecord(node);
-        var tid = trec.get('idplayer');
-        var sids = [];
+        var trec = me.view.getRecord(node),
+            tid = trec.get('idplayer'),
+            sids = [];
+
         Ext.each(data.records, function(r) {
             sids.push(r.get('idplayer'));
         });
@@ -913,16 +886,18 @@ Ext.define('SoL.module.Players.MergeTarget', {
     },
 
     notifyDrop: function(source, e, data) {
-        var me = this;
+        var me = this,
+            r = me._allowDrop(source, e, data);
 
-        var r = me._allowDrop(source, e, data);
-        if(r===false) {
+        if(r === false) {
             return false;
         }
-        var tdesc = SoL.module.Players.full_name(r[0]);
-        var tid = r[0].get('idplayer');
-        var sdescs = [];
-        var sids = [];
+
+        var tdesc = SoL.module.Players.full_name(r[0]),
+            tid = r[0].get('idplayer'),
+            sdescs = [],
+            sids = [];
+
         Ext.each(data.records, function(p) {
             sdescs.push(SoL.module.Players.full_name(p));
             sids.push(p.get('idplayer'));
@@ -1007,24 +982,25 @@ Ext.define('SoL.module.MyPlayers', {
     launcherText: null, // don't show an entry in the start menu
 
     init: function() {
-        var me = this;
-        var user = me.app.user;
+        var me = this,
+            user = me.app.user;
+
         if(!me.config.orig_dataURL) me.config.orig_dataURL = me.config.dataURL;
         me.config.dataURL = Ext.String.urlAppend(me.config.orig_dataURL,
-                                                 'filter_idowner=' + (user.is_admin
-                                                                      ? 'NULL'
-                                                                      : user.user_id));
+                                                 'filter_by_idowner=' + (user.is_admin
+                                                                         ? 'NULL'
+                                                                         : user.user_id));
         me.windowTitle = Ext.String.format(_('Players managed by {0}'), user.fullname);
         me.callParent();
     }
 });
 
 
-var phoneNumberRE = /^\+?([0-9]+ ?[0-9]+)*$/;
+var nonEmptyValueRE = /\S/;
 Ext.apply(Ext.form.field.VTypes, {
-    phone: function(val, field) {
-        return phoneNumberRE.test(val);
+    nonempty: function(val, field) {
+        return nonEmptyValueRE.test(val);
     },
-    phoneText: _('This field should be a telephone number in the format "+39 123 456 7890"'),
-    phoneMask: /[+\d ]/
+    nonemptyText: _(
+        'This field cannot be empty, must contain at least one non-space character')
 });
