@@ -1,8 +1,9 @@
-//-*- coding: utf-8 -*-
-//:Progetto:  SoL -- The ratings window
-//:Creato:    gio 05 dic 2013 20:31:44 CET
-//:Autore:    Lele Gaifax <lele@metapensiero.it>
-//:Licenza:   GNU General Public License version 3 or later
+// -*- coding: utf-8 -*-
+// :Project:   SoL -- The ratings window
+// :Created:   gio 05 dic 2013 20:31:44 CET
+// :Author:    Lele Gaifax <lele@metapensiero.it>
+// :License:   GNU General Public License version 3 or later
+// :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020, 2021 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -25,16 +26,18 @@ Ext.define('SoL.module.Ratings.Actions', {
         RECOMPUTE_RATING_ACTION: 'recompute_rating',
         DOWNLOAD_TOURNEYS_ACTION: 'download_tourneys',
         SHOW_TOURNEYS_ACTION: 'show_tourneys',
-        SHOW_PLAYERS_ACTION: 'show_players'
+        SHOW_PLAYERS_ACTION: 'show_players',
+        SHOW_RANKING_ACTION: 'show_ranking',
+        SHOW_LIT_PAGE_ACTION: 'show_lit'
     },
 
     initActions: function() {
-        var me = this;
-        var ids = me.statics();
+        var me = this,
+            ids = me.statics();
 
         me.callParent();
 
-        me.editRatingAction = me.addAction(new Ext.Action({
+        me.editRating = me.addAction(new Ext.Action({
             itemId: ids.EDIT_RATING_ACTION,
             text: _('Modify'),
             tooltip: _('Edit selected rating.'),
@@ -47,7 +50,7 @@ Ext.define('SoL.module.Ratings.Actions', {
             }
         }));
 
-        me.showTourneysAction = me.addAction(new Ext.Action({
+        me.showTourneys = me.addAction(new Ext.Action({
             itemId: ids.SHOW_TOURNEYS_ACTION,
             text: _('Tourneys'),
             tooltip: _('Show tourneys related to the selected rating.'),
@@ -55,15 +58,16 @@ Ext.define('SoL.module.Ratings.Actions', {
             disabled: true,
             needsOneSelectedRow: true,
             handler: function() {
-                var record = me.component.getSelectionModel().getSelection()[0];
-                var idrating = record.get('idrating');
-                var rating = record.get('description');
-                var module = me.module.app.getModule('tourneys-win');
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    idrating = record.get('idrating'),
+                    rating = record.get('description'),
+                    module = me.module.app.getModule('tourneys-win');
+
                 module.createOrShowWindow('ratings', idrating, rating);
             }
         }));
 
-        me.showPlayersAction = me.addAction(new Ext.Action({
+        me.showPlayers = me.addAction(new Ext.Action({
             itemId: ids.SHOW_PLAYERS_ACTION,
             text: _('Players'),
             tooltip: _('Show rated players in the selected rating.'),
@@ -71,15 +75,16 @@ Ext.define('SoL.module.Ratings.Actions', {
             disabled: true,
             needsOneSelectedRow: true,
             handler: function() {
-                var record = me.component.getSelectionModel().getSelection()[0];
-                var idrating = record.get('idrating');
-                var rating = record.get('description');
-                var module = me.module.app.getModule('rated-players-win');
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    idrating = record.get('idrating'),
+                    rating = record.get('description'),
+                    module = me.module.app.getModule('rated-players-win');
+
                 module.createOrShowWindow(idrating, rating);
             }
         }));
 
-        me.recomputeRatingAction = me.addAction(new Ext.Action({
+        me.recomputeRating = me.addAction(new Ext.Action({
             itemId: ids.RECOMPUTE_RATING_ACTION,
             text: _('Recompute'),
             tooltip: _('Recompute the selected rating from scratch.'),
@@ -87,8 +92,8 @@ Ext.define('SoL.module.Ratings.Actions', {
             disabled: true,
             needsOneSelectedRow: true,
             handler: function() {
-                var record = me.component.getSelectionModel().getSelection()[0];
-                var idrating = record.get('idrating');
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    idrating = record.get('idrating');
 
                 Ext.create("MP.window.Notification", {
                     position: 'br',
@@ -135,7 +140,7 @@ Ext.define('SoL.module.Ratings.Actions', {
             }
         }));
 
-        me.downloadTourneysAction = me.addAction(new Ext.Action({
+        me.downloadTourneys = me.addAction(new Ext.Action({
             itemId: ids.DOWNLOAD_TOURNEYS_ACTION,
             text: _('Download'),
             tooltip: _('Download all tourneys related to this rating.'),
@@ -143,9 +148,43 @@ Ext.define('SoL.module.Ratings.Actions', {
             disabled: true,
             needsOneSelectedRow: true,
             handler: function() {
-                var record = me.component.getSelectionModel().getSelection()[0];
-                var idrating = record.get('idrating');
-                var url = '/bio/dump?idrating=' + idrating;
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    idrating = record.get('idrating'),
+                    url = '/bio/dump?idrating=' + idrating;
+
+                window.open(url, "_blank");
+            }
+        }));
+
+        me.showRatingRanking = me.addAction(new Ext.Action({
+            itemId: ids.SHOW_RANKING_ACTION,
+            text: _('Ranking'),
+            tooltip: _('Print this rating ranking.'),
+            iconCls: 'print-icon',
+            disabled: true,
+            needsOneSelectedRow: true,
+            needsCleanStore: true,
+            handler: function() {
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    idrating = record.get('idrating'),
+                    url = '/pdf/ratingranking/' + idrating;
+
+                window.open(url, "_blank");
+            }
+        }));
+
+        me.showLitPage = me.addAction(new Ext.Action({
+            itemId: ids.SHOW_LIT_PAGE_ACTION,
+            text: _('Lit page'),
+            tooltip: _('Show the corresponding Lit page.'),
+            iconCls: 'lit-icon',
+            disabled: true,
+            needsOneSelectedRow: true,
+            handler: function() {
+                var record = me.component.getSelectionModel().getSelection()[0],
+                    guid = record.get('guid'),
+                    url = '/lit/rating/' + guid;
+
                 window.open(url, "_blank");
             }
         }));
@@ -159,17 +198,18 @@ Ext.define('SoL.module.Ratings.Actions', {
         var tbar = me.component.child('#ttoolbar');
 
         tbar.add(2, ' ',
-                 me.editRatingAction,
-                 me.showTourneysAction,
-                 me.showPlayersAction,
-                 me.recomputeRatingAction,
-                 me.downloadTourneysAction
+                 me.editRating,
+                 me.showTourneys,
+                 me.showPlayers,
+                 me.recomputeRating,
+                 me.downloadTourneys,
+                 me.showRatingRanking
         );
 
         me.component.on({
             itemdblclick: function() {
-                if(!me.editRatingAction.isDisabled())
-                    me.editRatingAction.execute();
+                if(!me.editRating.isDisabled())
+                    me.editRating.execute();
             }
         });
         me.component.store.on({
@@ -182,10 +222,10 @@ Ext.define('SoL.module.Ratings.Actions', {
     },
 
     shouldDisableAction: function(act) {
-        var me = this;
-        var disable = me.component.shouldDisableAction(act);
-        var statics = me.statics();
-        var currentuser = me.module.app.user;
+        var me = this,
+            disable = me.component.shouldDisableAction(act),
+            statics = me.statics(),
+            currentuser = me.module.app.user;
 
         if(!disable) {
             var record = me.component.getSelectionModel().getSelection()[0];
@@ -202,8 +242,9 @@ Ext.define('SoL.module.Ratings.Actions', {
                     // Disable pointless actions on historical ratings
                     if(record.get('level') == '0')
                         disable = true;
-                    else if(!currentuser.is_admin &&
-                            record.get('idowner') != currentuser.user_id)
+                    else if(!currentuser.is_admin
+                            && record.get('idowner') !== null
+                            && record.get('idowner') !== currentuser.user_id)
                         disable = true;
                     break;
 
@@ -215,11 +256,11 @@ Ext.define('SoL.module.Ratings.Actions', {
     },
 
     showEditRatingWindow: function(record) {
-        var me = this;
-        var desktop = me.module.app.getDesktop();
-        var win = desktop.getWindow('edit-rating-win');
-        var currentuser = me.module.app.user;
-        var canchange = currentuser.is_admin;
+        var me = this,
+            desktop = me.module.app.getDesktop(),
+            win = desktop.getWindow('edit-rating-win'),
+            currentuser = me.module.app.user,
+            canchange = currentuser.is_admin;
 
         // If the window is already present, destroy and recreate it,
         // to reapply configuration and filters
@@ -227,18 +268,18 @@ Ext.define('SoL.module.Ratings.Actions', {
             win.destroy();
         }
 
-        var metadata = me.module.config.metadata;
-        var size = desktop.getReasonableWindowSize(800, 440);
-        var editors = metadata.editors({
-            '*': { editor: MP.form.Panel.getDefaultEditorSettingsFunction() },
-            tau: { editor: { disabled: !canchange } },
-            default_rate: { editor: { disabled: !canchange } },
-            default_deviation: { editor: { disabled: !canchange } },
-            default_volatility: { editor: { disabled: !canchange } },
-            lower_rate: { editor: { disabled: !canchange } },
-            higher_rate: { editor: { disabled: !canchange } },
-            outcomes: { editor: { disabled: !canchange } }
-        });
+        var metadata = me.module.config.metadata,
+            size = desktop.getReasonableWindowSize(800, 440),
+            editors = metadata.editors({
+                '*': { editor: MP.form.Panel.getDefaultEditorSettingsFunction() },
+                tau: { editor: { disabled: !canchange } },
+                default_rate: { editor: { disabled: !canchange } },
+                default_deviation: { editor: { disabled: !canchange } },
+                default_volatility: { editor: { disabled: !canchange } },
+                lower_rate: { editor: { disabled: !canchange } },
+                higher_rate: { editor: { disabled: !canchange } },
+                outcomes: { editor: { disabled: !canchange } }
+            });
 
         // Remove the level 0, which cannot be assigned explicitly
         editors.level.store.splice(0, 1);
@@ -321,15 +362,17 @@ Ext.define('SoL.module.Ratings.Actions', {
                 type: 'help',
                 tooltip: _('Show user manual section.'),
                 callback: function() {
-                    var whsize = desktop.getReasonableWindowSize(800, 640);
-                    var wh = Ext.create('SoL.window.Help', {
-                        width: whsize.width,
-                        height: whsize.height,
-                        // TRANSLATORS: this is the URL of the manual
-                        // page explaining championship insert/edit
-                        help_url: _('/static/manual/en/ratings.html#insert-and-edit'),
-                        title: _('Help on ratings insert/edit')
-                    });
+                    var whsize = desktop.getReasonableWindowSize(800, 640),
+                        wh = Ext.create('SoL.window.Help', {
+                            width: whsize.width,
+                            height: whsize.height,
+                            // TRANSLATORS: this is the URL of the manual page explaining
+                            // ratings insert/edit, do not change unless the manual is actually
+                            // translated in the target language
+                            help_url: _('/static/manual/en/ratings.html#insert-and-edit'),
+                            title: _('Help on ratings insert/edit')
+                        });
+
                     wh.show();
                 }
             }]
@@ -348,8 +391,8 @@ Ext.define('SoL.module.Ratings', {
         'MP.grid.Panel'
     ],
     uses: [
-        'SoL.module.Players',
         'SoL.module.Ratings.Actions',
+        'SoL.module.Users',
         'SoL.window.Help'
     ],
 
@@ -377,8 +420,8 @@ Ext.define('SoL.module.Ratings', {
     },
 
     getConfig: function(callback) {
-        var me = this;
-        var cfg = me.config;
+        var me = this,
+            cfg = me.config;
 
         if(!cfg.metadata) {
             MP.data.MetaData.fetch(cfg.dataURL, me, function(metadata) {
@@ -397,7 +440,7 @@ Ext.define('SoL.module.Ratings', {
                     plugins: [
                         Ext.create('SoL.module.Ratings.Actions',
                                    { module: me }),
-                        Ext.create('SoL.module.Players.AssignOwnership',
+                        Ext.create('SoL.module.Users.AssignOwnership',
                                    { module: me })
                     ]
                 });
@@ -409,10 +452,11 @@ Ext.define('SoL.module.Ratings', {
         }
     },
 
-    createOrShowWindow: function() {
-        var me = this;
-        var desktop = me.app.getDesktop();
-        var win = desktop.getWindow(me.id);
+    createOrShowWindow: function(caller, id, description) {
+        var me = this,
+            config = me.config,
+            desktop = me.app.getDesktop(),
+            win = desktop.getWindow(me.id);
 
         // If the window is already present, destroy and recreate it,
         // to reapply configuration and filters
@@ -423,7 +467,8 @@ Ext.define('SoL.module.Ratings', {
         me.configure(
             [me.getConfig],
             function(done) {
-                var size = desktop.getReasonableWindowSize(750, 421, "S");
+                var size = desktop.getReasonableWindowSize(850, 421, "S"),
+                    filteredOn = '';
 
                 me.config.newRecordData = {
                     tau: 0.5,
@@ -432,12 +477,26 @@ Ext.define('SoL.module.Ratings', {
                     default_volatility: 0.06
                 };
 
+                if(caller == 'users') {
+                    filteredOn = ' (' + Ext.String.format(
+                        // TRANSLATORS: this is the explanation on the window title when the
+                        // filter is a user
+                        _('owned by {0}'), description) + ')';
+                    config = Ext.apply({
+                        filters: [{
+                            property: 'idowner',
+                            value: id,
+                            operator: '='
+                        }]
+                    }, config);
+                }
+
                 win = desktop.createWindow({
                     id: me.id,
-                    title: me.windowTitle || me.getLauncherText(),
+                    title: (me.windowTitle || me.getLauncherText()) + filteredOn,
                     taskbuttonTooltip: me.getLauncherTooltip(),
                     iconCls: me.iconCls,
-                    items: [me.config],
+                    items: [config],
                     x: size.x,
                     y: size.y,
                     width: size.width,
@@ -450,8 +509,9 @@ Ext.define('SoL.module.Ratings', {
                             var wh = Ext.create('SoL.window.Help', {
                                 width: whsize.width,
                                 height: whsize.height,
-                                // TRANSLATORS: this is the URL of the manual
-                                // page explaining ratings management
+                                // TRANSLATORS: this is the URL of the manual page explaining
+                                // ratings management, do not change unless the manual is
+                                // actually translated in the target language
                                 help_url: _('/static/manual/en/ratings.html'),
                                 title: _('Help on ratings management')
                             });
@@ -479,19 +539,21 @@ Ext.define('SoL.module.Ratings', {
     },
 
     shouldDisableDeleteAction: function() {
-        var grid = this;
-        var sm = grid.getSelectionModel();
-        var currentuser = grid.up().up().app.user;
+        var grid = this,
+            sm = grid.getSelectionModel(),
+            currentuser = grid.up().up().app.user;
 
         if(sm.getCount() > 0) {
-            var selrecs = sm.getSelection();
-            var disable = false;
+            var selrecs = sm.getSelection(),
+                disable = false;
 
             for(var i=selrecs.length-1; i>=0; i--) {
                 var record = selrecs[i];
 
                 if(record.get('Tourneys') > 0 ||
-                   (!currentuser.is_admin && currentuser.user_id != record.get('idowner'))) {
+                   (!currentuser.is_admin
+                    && record.get('idowner') !== null
+                    && currentuser.user_id !== record.get('idowner'))) {
                     disable = true;
                     break;
                 }
@@ -512,13 +574,14 @@ Ext.define('SoL.module.MyRatings', {
     launcherText: null, // don't show an entry in the start menu
 
     init: function() {
-        var me = this;
-        var user = me.app.user;
+        var me = this,
+            user = me.app.user;
+
         if(!me.config.orig_dataURL) me.config.orig_dataURL = me.config.dataURL;
         me.config.dataURL = Ext.String.urlAppend(me.config.orig_dataURL,
-                                                 'filter_idowner=' + (user.is_admin
-                                                                      ? 'NULL'
-                                                                      : user.user_id));
+                                                 'filter_by_idowner=' + (user.is_admin
+                                                                         ? 'NULL'
+                                                                         : user.user_id));
         me.windowTitle = Ext.String.format(_('Ratings managed by {0}'), user.fullname);
         me.callParent();
     }
