@@ -1,8 +1,9 @@
 // -*- coding: utf-8 -*-
-// :Progetto:  SoL -- Tourney management window
-// :Creato:    gio 20 nov 2008 18:21:20 CET
-// :Autore:    Lele Gaifax <lele@metapensiero.it>
-// :Licenza:   GNU General Public License version 3 or later
+// :Project:   SoL -- Tourney management window
+// :Created:   gio 20 nov 2008 18:21:20 CET
+// :Author:    Lele Gaifax <lele@metapensiero.it>
+// :License:   GNU General Public License version 3 or later
+// :Copyright: © 2008, 2009, 2010, 2013, 2014, 2016, 2018, 2020, 2021 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -29,13 +30,13 @@ Ext.define('SoL.module.Tourney', {
     },
 
     createOrShowWindow: function(tourney) {
-        var me = this;
-        var desktop = me.app.getDesktop();
-        var win = desktop.getWindow(me.id);
-        var winWidth = desktop.getWidth();
-        var winHeight = desktop.getHeight() - desktop.taskbar.getHeight();
-        var currentuser = me.app.user;
-        var readonly = (!currentuser.is_admin &&
+        var me = this,
+            desktop = me.app.getDesktop(),
+            win = desktop.getWindow(me.id),
+            winWidth = desktop.getWidth(),
+            winHeight = desktop.getHeight() - desktop.taskbar.getHeight(),
+            currentuser = me.app.user,
+            readonly = (!currentuser.is_admin &&
                         tourney.get('idowner') != currentuser.user_id);
 
         // If the window is already present, destroy and recreate it,
@@ -50,7 +51,7 @@ Ext.define('SoL.module.Tourney', {
 
         me.configure(SoL.view.Tourney.configurators(), function(done, config) {
             var tfilter = {
-                filter_idtourney: me.tourney.idtourney
+                filter_by_idtourney: me.tourney.idtourney
             };
 
             Ext.apply(config.Boards, {
@@ -113,23 +114,32 @@ Ext.define('SoL.module.Tourney', {
                     type: 'help',
                     tooltip: _('Show user manual section.'),
                     callback: function() {
-                        var whsize = desktop.getReasonableWindowSize(800, 640);
-                        var wh = Ext.create('SoL.window.Help', {
-                            width: whsize.width,
-                            height: whsize.height,
-                            // TRANSLATORS: this is the URL of the manual
-                            // page explaining tourney management
-                            help_url: _('/static/manual/en/tourney.html'),
-                            title: _('Help on tourney management')
-                        });
+                        var whsize = desktop.getReasonableWindowSize(800, 640),
+                            wh = Ext.create('SoL.window.Help', {
+                                width: whsize.width,
+                                height: whsize.height,
+                                // TRANSLATORS: this is the URL of the manual page explaining
+                                // tourney management, do not change unless the manual is
+                                // actually translated in the target language
+                                help_url: _('/static/manual/en/tourney.html'),
+                                title: _('Help on tourney management')
+                            });
+
                         wh.show();
                     }
+                }, {
+                    type: 'pin',
+                    tooltip: _('Show the corresponding Lit page.'),
+                    callback: function() {
+                        var url = '/lit/tourney/' + me.tourney.guid;
+                        window.open(url, "_blank");
+                    }
                 }]
             });
 
-            var mgrid = me.matches_grid = win.query('editable-grid[dataURL=/tourney/matches]')[0];
-            var rgrid = win.query('editable-grid[dataURL=/tourney/ranking]')[0];
-            var cview = win.query('competitors-dataview')[0];
+            var mgrid = me.matches_grid = win.query('editable-grid[dataURL=/tourney/matches]')[0],
+                rgrid = win.query('editable-grid[dataURL=/tourney/ranking]')[0],
+                cview = win.query('competitors-dataview')[0];
 
             rgrid.store.on('beforeload', function(store, operation) {
                 // remove the sorters, we do local sort...
@@ -138,8 +148,9 @@ Ext.define('SoL.module.Tourney', {
             });
 
             var updtitle = function(store) {
-                var nrecs = store.data.length;
-                var eastp = cview.ownerCt;
+                var nrecs = store.data.length,
+                    eastp = cview.ownerCt;
+
                 eastp.setTitle(Ext.String.format(_('Competitors ({0})'),
                                                  nrecs));
             };
@@ -154,7 +165,7 @@ Ext.define('SoL.module.Tourney', {
                     win.on('show', function() {
                         done();
                         rgrid.store.on('load', function() {
-                            if(me.tourney.rankedturn > 0) {
+                            if(me.tourney.rankedturn > 0 || me.tourney.GeneratedTurns) {
                                 cview.ownerCt.collapse();
                                 rgrid.expand();
                             } else {
@@ -171,6 +182,7 @@ Ext.define('SoL.module.Tourney', {
                 load: function(s, recs, opts) {
                     if(!mgrid.focusedCompetitor) {
                         var ordinalp = SoL.view.Matches.ordinalp;
+
                         if(Ext.isEmpty(recs)) {
                             mgrid.setTitle(Ext.String.format(_('Matches')));
                         } else if(recs[0].get('final')) {
@@ -186,7 +198,7 @@ Ext.define('SoL.module.Tourney', {
 
 
             cview.store.on('load', function(s, recs) {
-                me.tourney.partecipants = recs.length;
+                me.tourney.participants = recs.length;
                 mgrid.updateActions();
             });
 
@@ -214,8 +226,8 @@ Ext.define('SoL.module.Tourney', {
 
             me.togglePlayerDetail = function(grid, record) {
                 //jsl:unused grid
-                var idc = record.get('idcompetitor');
-                var mstore = mgrid.store;
+                var idc = record.get('idcompetitor'),
+                    mstore = mgrid.store;
 
                 if(mstore.isModified()) {
                     // Don't do anything if the user is inserting round' scores
@@ -255,19 +267,20 @@ Ext.define('SoL.module.Tourney', {
                     mstore.filter({
                         id: 'turn',
                         property: 'turn',
-                        value: turn
+                        value: turn,
+                        operator: '='
                     });
                     mgrid.focusedCompetitor = null;
                 }
             };
 
             if(!me.tourney.prized && !readonly) {
-                var newturn = mgrid.findActionById('new_turn');
-                var printbadges = cview.findActionById('print_badges');
-                var printfbadges = rgrid.findActionById('print_final_badges');
-                var printfcards = rgrid.findActionById('print_final_cards');
-                var rsave = rgrid.findActionById('save');
-                var rrestore = rgrid.findActionById('restore');
+                var newturn = mgrid.findActionById('new_turn'),
+                    printbadges = cview.findActionById('print_badges'),
+                    printfbadges = rgrid.findActionById('print_final_badges'),
+                    printfcards = rgrid.findActionById('print_final_cards'),
+                    rsave = rgrid.findActionById('save'),
+                    rrestore = rgrid.findActionById('restore');
 
                 cview.store.on('update', function(store, record, action) {
                     //jsl:unused store
@@ -296,12 +309,14 @@ Ext.define('SoL.module.Tourney', {
                 });
 
                 rgrid.store.on('load', function() {
-                    var prized = me.tourney.prized;
-                    var finals = me.tourney.finals;
+                    var prized = me.tourney.prized,
+                        finals = me.tourney.finals;
+
                     printbadges.setHidden(prized);
                     printfbadges.setHidden(!prized);
                     printfcards.setDisabled(!me.tourney.finalturns);
-                    printfcards.setHidden(finals===0
+                    printfcards.setHidden(me.tourney.system == 'knockout'
+                                          || finals===0
                                           || (finals && prized)
                                           || (!finals && !prized));
                     rsave.setHidden(!prized);
