@@ -1,8 +1,9 @@
 // -*- coding: utf-8 -*-
-// :Progetto:  SoL -- Ranking panel of the tourney management
-// :Creato:    gio 20 nov 2008 18:24:44 CET
-// :Autore:    Lele Gaifax <lele@metapensiero.it>
-// :Licenza:   GNU General Public License version 3 or later
+// :Project:   SoL -- Ranking panel of the tourney management
+// :Created:   gio 20 nov 2008 18:24:44 CET
+// :Author:    Lele Gaifax <lele@metapensiero.it>
+// :License:   GNU General Public License version 3 or later
+// :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2018, 2020 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -19,16 +20,18 @@ Ext.define('SoL.view.Ranking.Actions', {
         BY_NATIONALITY_ACTION: 'by_nationality',
         PRINT_RANKING_ACTION: 'print_ranking',
         PRINT_NATIONAL_RANKING_ACTION: 'print_national_ranking',
+        PRINT_WOMEN_RANKING_ACTION: 'print_women_ranking',
+        PRINT_UNDER_RANKING_ACTION: 'print_under_ranking',
         PRINT_FINAL_BADGES_ACTION: 'print_final_badges',
         PRINT_FINAL_CARDS_ACTION: 'print_final_cards',
         PRIZE_GIVING_ACTION: 'prize_giving'
     },
 
     initActions: function() {
-        var me = this;
-        var ids = me.statics();
-        var tourney = me.module.tourney;
-        var tprized = tourney.prized;
+        var me = this,
+            ids = me.statics(),
+            tourney = me.module.tourney,
+            tprized = tourney.prized;
 
         me.callParent();
 
@@ -48,8 +51,8 @@ Ext.define('SoL.view.Ranking.Actions', {
             tooltip: _('Print current ranking.'),
             iconCls: 'print-icon',
             handler: function() {
-                var turn = me.component.store.proxy.extraParams.turn;
-                var url = '/pdf/ranking/' + tourney.idtourney;
+                var turn = me.component.store.proxy.extraParams.turn,
+                    url = '/pdf/ranking/' + tourney.idtourney;
                 if(turn) url += '?turn=' + turn;
                 window.location.assign(url);
             }
@@ -61,13 +64,37 @@ Ext.define('SoL.view.Ranking.Actions', {
             tooltip: _('Print current national ranking.'),
             iconCls: 'print-icon',
             handler: function() {
-                var turn = me.component.store.proxy.extraParams.turn;
-                var url = '/pdf/nationalranking/' + tourney.idtourney;
+                var turn = me.component.store.proxy.extraParams.turn,
+                    url = '/pdf/nationalranking/' + tourney.idtourney;
                 if(turn) url += '?turn=' + turn;
                 window.location.assign(url);
             }
         }));
 
+        me.printWomenRankingAction = me.addAction(new Ext.Action({
+            itemId: ids.PRINT_WOMEN_RANKING_ACTION,
+            text: _('Women ranking'),
+            tooltip: _('Print women ranking.'),
+            iconCls: 'print-icon',
+            handler: function() {
+                var turn = me.component.store.proxy.extraParams.turn,
+                    url = '/pdf/womenranking/' + tourney.idtourney;
+                if(turn) url += '?turn=' + turn;
+                window.location.assign(url);
+            }
+        }));
+
+        me.printUnderRankingAction = me.addAction(new Ext.Action({
+            itemId: ids.PRINT_UNDER_RANKING_ACTION,
+            text: _('Junior ranking'),
+            tooltip: _('Print junior ranking.'),
+            iconCls: 'print-icon',
+            handler: function() {
+                var turn = me.component.store.proxy.extraParams.turn;
+                me.component.printUnderRanking(turn);
+            }
+        }));
+
         me.printFinalBadgesAction = me.addAction(new Ext.Action({
             itemId: ids.PRINT_FINAL_BADGES_ACTION,
             text: _('Badges'),
@@ -155,8 +182,17 @@ Ext.define('SoL.view.Ranking.Actions', {
                  me.printFinalCardsAction,
                  me.printFinalBadgesAction,
                  me.printRankingAction,
-                 me.printNationalRankingAction
-                );
+                 {
+                     text: _('Other…'),
+                     iconCls: 'print-icon',
+                     menu: {
+                         items: [
+                             me.printNationalRankingAction,
+                             me.printWomenRankingAction,
+                             me.printUnderRankingAction
+                         ]
+                     }
+                 });
 
         if(!me.module.tourney.finals)
             me.component.addDocked({
@@ -182,47 +218,47 @@ Ext.define('SoL.view.Ranking', {
     statics: {
         getConfig: function(callback, errorcb, config) {
             //jsl:unused errorcb
-            var me = this;
-            var ordinal = SoL.view.Matches.ordinal;
-            var cfg = config.Ranking = {
-                dataURL: '/tourney/ranking',
-                /*
-                features: [{
-                    ftype: 'groupingsummary',
-                    id: 'bynationgrouping',
-                    disabled: true,
-                    hideGroupedHeader: true,
-                    enableGroupingMenu: false,
-                    showSummaryRow: true,
-                    groupHeaderTpl: [
-                        "{[SoL.form.field.FlagsCombo.renderer(values.name)]}"
-                    ]
-                }],
-                remoteGroup: false,
-                groupers: 'player1Nationality',
-                */
-                header: true,
-                layout: 'fit',
-                noAddAndDelete: true,
-                noBottomToolbar: true,
-                noFilterbar: true,
-                pageSize: 999,
-                plugins: [
-                    Ext.create('SoL.view.Ranking.Actions', {
-                        module: me
-                    })
-                ],
-                remoteSort: false,
-                saveChangesURL: '/bio/saveChanges',
-                title: (config.tourney.rankedturn === 0
-                        ? _('Ranking')
-                        : (config.tourney.prized
-                           ? _('Final ranking')
-                           : Ext.String.format(
-                               _('Ranking after {0} round'),
-                               ordinal(config.tourney.rankedturn)))),
-                xtype: 'ranking-grid'
-            };
+            var me = this,
+                ordinal = SoL.view.Matches.ordinal,
+                cfg = config.Ranking = {
+                    dataURL: '/tourney/ranking',
+                    /*
+                      features: [{
+                      ftype: 'groupingsummary',
+                      id: 'bynationgrouping',
+                      disabled: true,
+                      hideGroupedHeader: true,
+                      enableGroupingMenu: false,
+                      showSummaryRow: true,
+                      groupHeaderTpl: [
+                      "{[SoL.form.field.FlagsCombo.renderer(values.name)]}"
+                      ]
+                      }],
+                      remoteGroup: false,
+                      groupers: 'player1Nationality',
+                    */
+                    header: true,
+                    layout: 'fit',
+                    noAddAndDelete: true,
+                    noBottomToolbar: true,
+                    noFilterbar: true,
+                    pageSize: 999,
+                    plugins: [
+                        Ext.create('SoL.view.Ranking.Actions', {
+                            module: me
+                        })
+                    ],
+                    remoteSort: false,
+                    saveChangesURL: '/bio/saveChanges',
+                    title: (config.tourney.rankedturn === 0
+                            ? _('Ranking')
+                            : (config.tourney.prized
+                               ? _('Final ranking')
+                               : Ext.String.format(
+                                   _('Ranking after {0} round'),
+                                   ordinal(config.tourney.rankedturn)))),
+                    xtype: 'ranking-grid'
+                };
 
             MP.data.MetaData.fetch(cfg.dataURL, me, function(metadata) {
                 var overrides = {
@@ -258,28 +294,30 @@ Ext.define('SoL.view.Ranking', {
     },
 
     initEvents: function() {
-        var me = this;
-        var module = me.module;
-        var tourney = module.tourney;
-        var ordinal = SoL.view.Matches.ordinal;
-        var normal_sorters = [
-            { property: 'prize', direction: 'DESC' },
-            { property: 'points', direction: 'DESC' },
-            { property: 'bucholz', direction: 'DESC' },
-            { property: 'netscore', direction: 'DESC' },
-            { property: 'totscore', direction: 'DESC' },
-            { property: 'rate', direction: 'DESC' },
-            { property: 'description', direction: 'ASC' }
-        ];
-        var newturn_sorters = [
-            { property: 'prize', direction: 'DESC' },
-            { property: 'points', direction: 'DESC' },
-            { property: 'bucholz', direction: 'DESC' },
-            { property: 'rate', direction: 'DESC' },
-            { property: 'netscore', direction: 'DESC' },
-            { property: 'totscore', direction: 'DESC' },
-            { property: 'description', direction: 'ASC' }
-        ];
+        var me = this,
+            module = me.module,
+            tourney = module.tourney,
+            ordinal = SoL.view.Matches.ordinal,
+            normal_sorters = [
+                { property: 'prize', direction: 'DESC' },
+                { property: 'points', direction: 'DESC' },
+                { property: 'bucholz', direction: 'DESC' },
+                { property: 'netscore', direction: 'DESC' },
+                { property: 'totscore', direction: 'DESC' },
+                { property: 'position', direction: 'ASC' },
+                { property: 'rate', direction: 'DESC' },
+                { property: 'description', direction: 'ASC' }
+            ],
+            newturn_sorters = [
+                { property: 'prize', direction: 'DESC' },
+                { property: 'points', direction: 'DESC' },
+                { property: 'bucholz', direction: 'DESC' },
+                { property: 'position', direction: 'ASC' },
+                { property: 'rate', direction: 'DESC' },
+                { property: 'netscore', direction: 'DESC' },
+                { property: 'totscore', direction: 'DESC' },
+                { property: 'description', direction: 'ASC' }
+            ];
 
         me.callParent();
 
@@ -288,8 +326,8 @@ Ext.define('SoL.view.Ranking', {
             return !tourney.readOnly && tourney.prized;
         });
         me.store.on('load', function(store) {
-            var pcol = me.getColumnByName('prize');
-            var turn = me.store.proxy.extraParams.turn;
+            var pcol = me.getColumnByName('prize'),
+                turn = me.store.proxy.extraParams.turn;
 
             if(tourney.prized && !turn) {
                 pcol.show();
@@ -330,9 +368,9 @@ Ext.define('SoL.view.Ranking', {
     },
 
     toggleView: function(action) {
-        var me = this;
-        var view = me.view;
-        var feature = view.getFeature('bynationgrouping');
+        var me = this,
+            view = me.view,
+            feature = view.getFeature('bynationgrouping');
 
         if(feature.disabled) {
             feature.enable();
@@ -341,5 +379,79 @@ Ext.define('SoL.view.Ranking', {
             feature.disable();
             action.setText(_('Normal'));
         }
+    },
+
+    printUnderRanking: function(turn) {
+        var me = this,
+            tourney = me.module.tourney,
+            url = '/pdf/underranking/' + tourney.idtourney,
+            win, form,
+            winWidth = 215,
+            winHeight = 110;
+
+        var handler = function() {
+            var frm = form.getForm();
+            if(frm.isValid()) {
+                var age = frm.getFields().items[0].getValue();
+                url += '?age=' + age;
+                if(turn)
+                    url += '&turn=' + turn;
+                win.destroy();
+                window.location.assign(url);
+            }
+        };
+
+        var onKeyDown = function(field, event) {
+            if (event.keyCode === event.RETURN || event.keyCode === 10) {
+                handler();
+            }
+        };
+
+        form = new Ext.form.Panel({
+            frame: true,
+            bodyPadding: '10 10 0',
+            defaults: {
+                labelWidth: 50,
+                anchor: '100%'
+            },
+            items: [{
+                xtype: 'numberfield',
+                itemId: 'age',
+                increment: 3,
+                allowBlank: false,
+                minValue: 5,
+                maxValue: 99,
+                value: 18,
+                enableKeyEvents: true,
+                listeners: {
+                    keydown: onKeyDown,
+                    scope: me
+                }
+
+            }],
+            buttons: [{
+                text: _('Cancel'),
+                handler: function() {
+                    win.destroy();
+                }
+            }, {
+                text: _('Confirm'),
+                formBind: true,
+                handler: handler
+            }]
+        });
+
+        win = me.module.app.getDesktop().createWindow({
+            title: _('Maximum age'),
+            width: winWidth,
+            height: winHeight,
+            layout: 'fit',
+            minimizable: false,
+            maximizable: false,
+            items: [form],
+            defaultFocus: 'age'
+        });
+
+        win.show();
     }
 });
