@@ -1,24 +1,27 @@
 # -*- coding: utf-8 -*-
-# :Progetto:  SoL -- Batched I/O controller
-# :Creato:    lun 09 feb 2009 10:32:22 CET
-# :Autore:    Lele Gaifax <lele@metapensiero.it>
-# :Licenza:   GNU General Public License version 3 or later
+# :Project:   SoL -- Batched I/O controller
+# :Created:   lun 09 feb 2009 10:32:22 CET
+# :Author:    Lele Gaifax <lele@metapensiero.it>
+# :License:   GNU General Public License version 3 or later
+# :Copyright: © 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020 Lele Gaifax
 #
 
+from io import BytesIO
 from datetime import date
 import logging
 
-import transaction
-
+from nacl.secret import SecretBox
 from pyramid.httpexceptions import HTTPBadRequest, HTTPInternalServerError
 from pyramid.settings import asbool
 from pyramid.view import view_config
+import transaction
+from translationstring import TranslationString
 
 from . import get_request_logger, unauthorized_for_guest
 from ..i18n import translatable_string as _, ngettext, translator
-from ..models import DBSession, Player, Rating, Championship, Tourney
+from ..models import Player, Rating, Championship, Tourney
 from ..models.bio import changes_logger, dump_sol, load_sol, restore, save_changes
-from ..models.errors import LoadError, OperationAborted
+from ..models.errors import OperationAborted
 from ..models.utils import njoin
 
 
@@ -33,18 +36,36 @@ def backup(request):
 
     get_request_logger(request, logger).info('Full backup')
 
+    settings = request.registry.settings
+    if settings.get('desktop.version') == 'test':
+        secret_key = request.params.get('secret_key', None)
+    else:  # pragma: no cover
+        secret_key = None
+
+    if secret_key is None:
+        secret_key = settings.get('sol.backup_secret_key')
+
     only_played_tourneys = asbool(request.params.get('only_played_tourneys', False))
     serialization_format = request.params.get('serialization_format', 'yaml')
-    settings = request.registry.settings
     response = request.response
 
-    response.body = backup(DBSession(),
-                           settings['sol.portraits_dir'],
-                           settings['sol.emblems_dir'],
-                           only_played_tourneys=only_played_tourneys,
-                           serialization_format=serialization_format)
-    response.content_type = 'application/zip'
-    filename = '%s.zip' % date.today().isoformat()
+    result = backup(request.dbsession,
+                    settings['sol.portraits_dir'],
+                    settings['sol.emblems_dir'],
+                    only_played_tourneys=only_played_tourneys,
+                    serialization_format=serialization_format)
+
+    if secret_key is not None:
+        box = SecretBox(bytes.fromhex(secret_key))
+        result = box.encrypt(result)
+        content_type = 'application/octet-stream'
+        filename = '%s.box' % date.today().isoformat()
+    else:
+        content_type = 'application/zip'
+        filename = '%s.zip' % date.today().isoformat()
+
+    response.body = result
+    response.content_type = content_type
     response.content_disposition = 'attachment; filename=%s' % filename
 
     return response
@@ -64,7 +85,7 @@ def dump(request):
     t = translator(request)
 
     try:
-        sess = DBSession()
+        sess = request.dbsession
 
         compress = asbool(params.get('gzip', not debug))
         ext = 'sol.gz' if compress else 'sol'
@@ -72,7 +93,7 @@ def dump(request):
         if 'idtourney' in params:
             idtourney = int(params['idtourney'])
             tourney = sess.query(Tourney).get(idtourney)
-            if tourney is None: # pragma: no cover
+            if tourney is None:  # pragma: no cover
                 raise HTTPBadRequest(
                     t(_('Tourney $idtourney does not exist',
                         mapping=dict(idtourney=idtourney))))
@@ -87,10 +108,10 @@ def dump(request):
                                        ext)
         elif 'idchampionship' in params:
             idchampionship = int(params['idchampionship'])
-            tourneys = sess.query(Tourney) \
-                       .filter_by(idchampionship=idchampionship) \
-                       .order_by(Tourney.date).all()
-            if not tourneys: # pragma: no cover
+            tourneys = (sess.query(Tourney)
+                        .filter_by(idchampionship=idchampionship)
+                        .order_by(Tourney.date)).all()
+            if not tourneys:  # pragma: no cover
                 raise HTTPBadRequest(
                     t(_('No tourneys in championship $idchampionship',
                         mapping=dict(idchampionship=idchampionship))))
@@ -100,11 +121,11 @@ def dump(request):
             filename = '%s.%s' % (sub(r'\W+', '_', sdesc), ext)
         elif 'idclub' in params:
             idclub = int(params['idclub'])
-            tourneys = sess.query(Tourney) \
-                       .select_from(join(Tourney, Championship)) \
-                       .filter(Championship.idclub == idclub) \
-                       .order_by(Tourney.date).all()
-            if not tourneys: # pragma: no cover
+            tourneys = (sess.query(Tourney)
+                        .select_from(join(Tourney, Championship))
+                        .filter(Championship.idclub == idclub)
+                        .order_by(Tourney.date)).all()
+            if not tourneys:  # pragma: no cover
                 raise HTTPBadRequest(
                     t(_('No tourneys organized by club $idclub',
                         mapping=dict(idclub=idclub))))
@@ -114,7 +135,7 @@ def dump(request):
             filename = '%s.%s' % (sub(r'\W+', '_', cdesc), ext)
         else:
             tourneys = sess.query(Tourney).order_by(Tourney.date).all()
-            if not tourneys:
+            if not tourneys:  # pragma: no cover
                 raise HTTPBadRequest(t(_('No tourneys at all')))
 
             filename = '%s.%s' % (date.today().isoformat(), ext)
@@ -130,47 +151,111 @@ def dump(request):
 
         return response
 
-    except HTTPBadRequest as e: # pragma: no cover
+    except HTTPBadRequest as e:  # pragma: no cover
         get_request_logger(request, logger).error("Couldn't dump tourney: %s", e)
         raise
 
-    except Exception as e: # pragma: no cover
+    except Exception as e:  # pragma: no cover
         get_request_logger(request, logger).critical("Couldn't dump tourney: %s", e,
                                                      exc_info=True)
         raise HTTPInternalServerError(str(e))
 
 
+def _recognizeIntegrityError(e, t, rlogger):  # pragma: nocover
+    # Catch most common reasons, ugly as it is
+    excmsg = str(e)
+    if ('columns date, idchampionship are not unique' in excmsg
+        or (' UNIQUE constraint failed:'
+            ' tourneys.date, tourneys.idchampionship') in excmsg):
+        rlogger.warning('Not allowing duplicated event: %s', excmsg)
+        message = t(_('There cannot be two tourneys of the same'
+                      ' championship on the same day, sorry!'))
+    elif (' UNIQUE constraint failed: championships.description,'
+          ' championships.idclub') in excmsg:
+        rlogger.warning('Not allowing duplicated championship: %s', excmsg)
+        message = t(_('There cannot be two championships with the same'
+                      ' description organized by the same club!'))
+    elif ' UNIQUE constraint failed: clubs.description' in excmsg:
+        rlogger.warning('Not allowing duplicated club: %s', excmsg)
+        message = t(_('There cannot be two clubs with the same description!'))
+    elif ' UNIQUE constraint failed: ratings.description' in excmsg:
+        rlogger.warning('Not allowing duplicated rating: %s', excmsg)
+        message = t(_('There cannot be two ratings with the same description!'))
+    elif ' UNIQUE constraint failed: championships.description' in excmsg:
+        rlogger.warning('Not allowing duplicated championship: %s', excmsg)
+        message = t(_('There cannot be two championships with the same description!'))
+    elif (' UNIQUE constraint failed: players.lastname, players.firstname,'
+          ' players.nickname' in excmsg or 'columns lastname, firstname, nickname'
+          ' are not unique' in excmsg):
+        rlogger.warning('Not allowing duplicated player: %s', excmsg)
+        message = t(_('There cannot be two players with the same'
+                      ' firstname, lastname and nickname!'))
+    elif (' UNIQUE constraint failed: matches.idtourney, matches.idcompetitor1,'
+          ' matches.idcompetitor2') in excmsg:
+        rlogger.warning('Not allowing duplicated match: %s', excmsg)
+        message = t(_('There cannot be two matches involving the same competitors!'))
+    elif (' UNIQUE constraint failed: rates.idrating, rates.idplayer, rates.date') in excmsg:
+        rlogger.warning('Not allowing duplicated rate: %s', excmsg)
+        message = t(_('There cannot be two rates of the same player in the same date!'))
+    elif (' may not be NULL' in excmsg or ' NOT NULL constraint failed' in excmsg):
+        rlogger.warning('Incomplete data: %s', excmsg)
+        message = t(_('Missing information prevents saving changes!'))
+        message += "<br/>"
+        message += t(_('Some mandatory fields were not filled in, please recheck.'))
+    else:  # pragma: no cover
+        rlogger.error('Could not save changes: %s', excmsg)
+        message = t(_('Integrity error prevents saving changes!'))
+        message += "<br/>"
+        message += t(_('Most probably a field contains an invalid value:'
+                       ' consult the application log for details.'))
+    # Explicit rollback, to avoid a pointless traceback
+    transaction.doom()
+    return message
+
+
 @view_config(route_name='merge_players', renderer='json')
 @unauthorized_for_guest
 def mergePlayers(request):
     "Merge several players into a single one."
 
+    from sqlalchemy.exc import IntegrityError, InvalidRequestError
+
+    t = translator(request)
+    rlogger = get_request_logger(request, logger)
+
     tid = int(request.params['tid'])
     sids = request.params.getall('sids')
-    if not isinstance(sids, list):
+    if not isinstance(sids, list):  # pragma: no cover
         sids = [sids]
     sids = [int(i) for i in sids]
+    if not sids:
+        return dict(success=False, message=t(_('Unspecified merge ids')))
 
-    sas = DBSession()
+    sas = request.dbsession
     player = sas.query(Player).get(tid)
+    if player is None:  # pragma: no cover
+        return dict(success=False, message=t(_('Invalid target player id')))
+
+    success = False
     try:
         replaced = player.mergePlayers(sids, get_request_logger(request, changes_logger))
     except OperationAborted as e:
-        msg = str(e)
-        get_request_logger(request, logger).error("Couldn't merge players: %s", msg)
-        success = False
-    except Exception as e:
-        msg = str(e)
-        get_request_logger(request, logger).exception("Couldn't merge players: %s", msg)
-        success = False
+        errormsg = e.args[0]
+        rlogger.error("Couldn't merge players: %s", errormsg)
+        message = t(errormsg)
+    except (IntegrityError, InvalidRequestError) as e:  # pragma: no cover
+        message = _recognizeIntegrityError(e, t, rlogger)
+    except Exception as e:  # pragma: no cover
+        message = str(e)
+        rlogger.exception("Couldn't merge players: %s", message)
     else:
         count = len(replaced)
-        msg = ngettext('$count player has been merged into $player',
-                       '$count players has been merged into $player',
-                       count, mapping=dict(count=count,
-                                           player=player.caption(False)))
+        message = ngettext('$count player has been merged into $player',
+                           '$count players has been merged into $player',
+                           count, mapping=dict(count=count,
+                                               player=player.caption(False)))
         success = True
-    return dict(success=success, message=msg)
+    return dict(success=success, message=message)
 
 
 @view_config(route_name='save_changes', renderer='json')
@@ -179,16 +264,16 @@ def saveChanges(request):
     """Save changes made to a set of records."""
 
     from sqlalchemy.exc import DBAPIError, IntegrityError, InvalidRequestError
-    from metapensiero.sqlalchemy.proxy.json import json2py
+    from metapensiero.sqlalchemy.proxy.json import JSON
 
     t = translator(request)
     rlogger = get_request_logger(request, logger)
 
     params = request.params
-    mr = json2py(params['modified_records'])
-    dr = json2py(params['deleted_records'])
+    mr = JSON.decode(params['modified_records'])
+    dr = JSON.decode(params['deleted_records'])
 
-    sess = DBSession()
+    sess = request.dbsession
 
     success = False
     try:
@@ -211,60 +296,23 @@ def saveChanges(request):
             rlogger.info('Changes successfully committed: %s',
                          njoin(infomsg, localized=False))
     except OperationAborted as e:
-        message = str(e)
-        rlogger.warning('Operation refused: %s', message)
+        errormsg = e.args[0]
+        rlogger.warning('Operation refused: %s', errormsg)
+        message = t(errormsg)
     except (IntegrityError, InvalidRequestError) as e:
-        # Catch most common reasons, ugly as it is
-        excmsg = str(e)
-        if ('columns date, idchampionship are not unique' in excmsg
-            or (' UNIQUE constraint failed:'
-                ' tourneys.date, tourneys.idchampionship') in excmsg):
-            rlogger.warning('Not allowing duplicated event: %s', excmsg)
-            message = t(_('There cannot be two tourneys of the same'
-                          ' championship on the same day, sorry!'))
-        elif (' UNIQUE constraint failed:'
-                ' championships.description, championships.idclub') in excmsg:
-            rlogger.warning('Not allowing duplicated championship: %s', excmsg)
-            message = t(_('There cannot be two championships with the same'
-                          ' description organized by the same club!'))
-        elif ' UNIQUE constraint failed: clubs.description' in excmsg:
-            rlogger.warning('Not allowing duplicated club: %s', excmsg)
-            message = t(_('There cannot be two clubs with the same description!'))
-        elif ' UNIQUE constraint failed: ratings.description' in excmsg:
-            rlogger.warning('Not allowing duplicated rating: %s', excmsg)
-            message = t(_('There cannot be two ratings with the same description!'))
-        elif ' UNIQUE constraint failed: championships.description' in excmsg:
-            rlogger.warning('Not allowing duplicated championship: %s', excmsg)
-            message = t(_('There cannot be two championships with the same description!'))
-        elif (' UNIQUE constraint failed: players.lastname, players.firstname,'
-              ' players.nickname' in excmsg or 'columns lastname, firstname, nickname'
-              ' are not unique' in excmsg):
-            rlogger.warning('Not allowing duplicated player: %s', excmsg)
-            message = t(_('There cannot be two players with the same'
-                          ' firstname, lastname and nickname!'))
-        elif (' may not be NULL' in excmsg
-              or ' NOT NULL constraint failed' in excmsg):
-            rlogger.warning('Incomplete data: %s', excmsg)
-            message = t(_('Missing information prevents saving changes!'))
-            message += "<br/>"
-            message += t(_('Some mandatory fields were not filled in, please recheck.'))
-        else:
-            rlogger.error('Could not save changes: %s', excmsg)
-            message = t(_('Integrity error prevents saving changes!'))
-            message += "<br/>"
-            message += t(_('Most probably a field contains an invalid value:'
-                           ' consult the application log for details.'))
-    except DBAPIError as e:
+        message = _recognizeIntegrityError(e, t, rlogger)
+    except DBAPIError as e:  # pragma: no cover
         rlogger.error('Could not save changes: %s', e)
         message = t(_('Error occurred while saving changes!'))
         message += "<br/>"
         message += t(_('Please inform the admin or consult the application log.'))
-    except Exception as e: # pragma: no cover
+    except Exception as e:  # pragma: no cover
         rlogger.critical('Could not save changes: %s', e, exc_info=True)
         message = t(_('Internal error!'))
         message += "<br/>"
         message += t(_('Please inform the admin or consult the application log.'))
-
+    if not success:
+        sess.rollback()
     return dict(success=success, message=message)
 
 
@@ -272,7 +320,7 @@ def saveChanges(request):
 def upload(request):
     "Handle the upload of tourneys data."
 
-    from metapensiero.sqlalchemy.proxy.json import py2json
+    from metapensiero.sqlalchemy.proxy.json import JSON
 
     t = translator(request)
 
@@ -289,7 +337,7 @@ def upload(request):
 
         response = request.response
         response.content_type = 'text/html'
-        response.text = py2json(dict(success=False, message=msg))
+        response.text = JSON.encode(dict(success=False, message=msg))
 
         return response
 
@@ -302,17 +350,34 @@ def upload(request):
     if archive is not None:
         fnendswith = archive.filename.endswith
 
-        if fnendswith('.sol') or fnendswith('.sol.gz'):
+        if fnendswith(('.sol', '.sol.gz')):
             load = load_sol
-        elif fnendswith('.zip'):
+        elif fnendswith(('.box', '.zip')):
             if request.session['is_admin']:
-                load = lambda sasess, url, content, idowner: restore(
-                    sasess,
-                    settings['sol.portraits_dir'],
-                    settings['sol.emblems_dir'],
-                    url,
-                    content,
-                    idowner)
+                def load(sasess, url, content, idowner):
+                    if settings.get('desktop.version') == 'test':
+                        secret_key = request.params.get('secret_key', None)
+                    else:  # pragma: no cover
+                        secret_key = None
+
+                    if secret_key is None:
+                        secret_key = settings.get('sol.backup_secret_key')
+
+                    if secret_key is not None:
+                        box = SecretBox(bytes.fromhex(secret_key))
+                        try:
+                            content = box.decrypt(content.read())
+                        except Exception:  # pragma: no cover
+                            raise OperationAborted(_("Unallowed, wrong encryption!"))
+                        else:
+                            content = BytesIO(content)
+
+                    return restore(sasess,
+                                   settings['sol.portraits_dir'],
+                                   settings['sol.emblems_dir'],
+                                   url,
+                                   content,
+                                   idowner)
             else:
                 msg = t(_('Only admin can restore whole ZIPs, sorry!'))
                 get_request_logger(request, logger).warning('Attempt to restore %s rejected',
@@ -326,17 +391,19 @@ def upload(request):
         msg = t(_('Required "archive" parameter is missing!'))
 
     if load is not None:
-        sas = DBSession()
+        sas = request.dbsession
         try:
             with transaction.manager:
                 tourneys, skipped = load(sas, url=archive.filename, content=archive.file,
                                          idowner=idowner)
-        except (LoadError, OperationAborted) as e: # pragma: no cover
-            msg = str(e)
+        except OperationAborted as e:  # pragma: no cover
+            msg = e.args[0]
+            if isinstance(msg, TranslationString):
+                msg = t(msg)
             success = False
             get_request_logger(request, logger).error('Upload of %s failed: %s',
                                                       archive.filename, msg)
-        except Exception as e: # pragma: no cover
+        except Exception as e:  # pragma: no cover
             transaction.abort()
             success = False
             msg = t(_('Upload of $file failed: $error',
@@ -363,7 +430,7 @@ def upload(request):
 
     response = request.response
     response.content_type = 'text/html'
-    response.text = py2json(dict(success=success, message=msg))
+    response.text = JSON.encode(dict(success=success, message=msg))
 
     return response
 
@@ -377,16 +444,16 @@ def recomputeRating(request):
 
     rid = int(request.params['idrating'])
 
-    sas = DBSession()
+    sas = request.dbsession
     rating = sas.query(Rating).get(rid)
-    if rating is None:
+    if rating is None:  # pragma: no cover
         raise HTTPBadRequest(
             t(_('Rating $idrating does not exist',
                 mapping=dict(idrating=rid))))
     else:
         try:
             rating.recompute(scratch=True)
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             msg = str(e)
             get_request_logger(request, logger).exception("Couldn't recompute rating: %s", msg)
             success = False
